# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.utils.logger_utils import LoggerUtils
import json
import requests
import sys
import traceback


class ComplexEncoder(json.JSONEncoder):
    def default(self, obj):
        if hasattr(obj, 'repr_json'):
            return obj.repr_json()
        else:
            return json.JSONEncoder.default(self, obj)

class HttpRequest:
    @staticmethod
    def post_json(url, data):
        """
        :param url:
        :param data: dict like
        :return:
        """
        headers = {'Content-type': 'application/json; charset=utf-8'}
        r = requests.post(url, json=data, headers=headers)
        try:
            response = json.loads(r.text)
            if 'success' not in response or not response['success']:
                LoggerUtils().get_logger().error('[Intraday] post signals not success')
        except Exception as e:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            LoggerUtils().get_logger().error(repr(traceback.format_exception(exc_type, exc_value, exc_traceback)))
            LoggerUtils().get_logger().error('[Intraday] post error')
        return

    @staticmethod
    def get_json(resp):
        resp.headers['Content-type'] = 'application/json;charset=utf-8'
        resp.headers['Access-Control-Allow-Origin'] = '*'
        return resp


# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.orm.hbdb import HBDB
from datetime import datetime, timedelta
from matplotlib.ticker import FuncFormatter
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False
sns.set_style('white', {'font.sans-serif': ['simhei', 'Arial']})


def to_percent(temp, position):
    return '%1.0f'%(temp) + '%'

def get_cal_and_trade_cal(start, end):
    """
    获取交易日期
    """
    cal = HBDB().read_cal(start, end)
    cal = cal.rename(columns={'JYRQ': 'TRADE_DATE', 'SFJJ': 'IS_OPEN', 'SFZM': 'IS_WEEK_END', 'SFYM': 'IS_MONTH_END'})
    cal['IS_OPEN'] = cal['IS_OPEN'].astype(int).replace({0: 1, 1: 0})
    cal['IS_WEEK_END'] = cal['IS_WEEK_END'].fillna(0).astype(int)
    cal['IS_MONTH_END'] = cal['IS_MONTH_END'].fillna(0).astype(int)
    cal = cal.sort_values('TRADE_DATE')
    trade_cal = cal[cal['IS_OPEN'] == 1]
    trade_cal['RECENT_TRADE_DATE'] = trade_cal['TRADE_DATE']
    trade_cal['PREV_TRADE_DATE'] = trade_cal['TRADE_DATE'].shift(1)
    trade_cal = trade_cal[['TRADE_DATE', 'RECENT_TRADE_DATE', 'PREV_TRADE_DATE', 'IS_OPEN', 'IS_WEEK_END', 'IS_MONTH_END']]
    cal = cal.merge(trade_cal[['TRADE_DATE', 'RECENT_TRADE_DATE']], on=['TRADE_DATE'], how='left')
    cal['RECENT_TRADE_DATE'] = cal['RECENT_TRADE_DATE'].fillna(method='ffill')
    cal = cal.merge(trade_cal[['TRADE_DATE', 'PREV_TRADE_DATE']], on=['TRADE_DATE'], how='left')
    cal['PREV_TRADE_DATE'] = cal['PREV_TRADE_DATE'].fillna(method='bfill')
    cal = cal[['TRADE_DATE', 'RECENT_TRADE_DATE', 'PREV_TRADE_DATE', 'IS_OPEN', 'IS_WEEK_END', 'IS_MONTH_END']]
    return cal, trade_cal

def get_industry_index(date, stock_industry):
    trade_cal = get_cal_and_trade_cal('20000101', datetime.today().strftime('%Y%m%d'))[1]
    trade_cal['TRADE_DATE'] = trade_cal['TRADE_DATE'].astype(str)
    trade_cal = trade_cal.sort_values('TRADE_DATE')
    recent_trade_date = trade_cal[trade_cal['TRADE_DATE'] <= date]['TRADE_DATE'].iloc[-1]

    stock_finance = HBDB().read_finance_data_given_date(date)
    stock_finance.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/stock_finance_{0}.hdf'.format(date), key='table', mode='w')
    stock_finance = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/stock_finance_{0}.hdf'.format(date), key='table')
    stock_finance = stock_finance[['TICKER_SYMBOL', 'END_DATE', 'PUBLISH_DATE', 'ROE_TTM', 'GROSS_INCOME_RATIO_TTM', 'NET_PROFIT', 'INCOME_PS']]
    stock_finance = stock_finance.sort_values(['TICKER_SYMBOL', 'END_DATE', 'PUBLISH_DATE']).drop_duplicates(['TICKER_SYMBOL', 'END_DATE'], keep='last')
    stock_mv = HBDB().read_stock_market_value_given_date(recent_trade_date)
    stock_mv.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/stock_mv_{0}.hdf'.format(date), key='table', mode='w')
    stock_mv = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/stock_mv_{0}.hdf'.format(date), key='table')
    stock_mv = stock_mv[['TICKER_SYMBOL', 'MARKET_VALUE']]
    stock_price = HBDB().read_stock_daily_k_given_date(recent_trade_date)
    stock_price.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/stock_price_{0}.hdf'.format(date), key='table', mode='w')
    stock_price = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/stock_price_{0}.hdf'.format(date), key='table')
    stock_price = stock_price[['TICKER_SYMBOL', 'CLOSE_PRICE']]
    stock_finance = stock_finance.merge(stock_mv, on=['TICKER_SYMBOL'], how='left').merge(stock_price, on=['TICKER_SYMBOL'], how='left')
    stock_finance['TOTAL_SHARES'] = stock_finance['MARKET_VALUE'] / stock_finance['CLOSE_PRICE']
    stock_finance['INCOME'] = stock_finance['INCOME_PS'] * stock_finance['TOTAL_SHARES']
    stock_finance = stock_finance[['TICKER_SYMBOL', 'END_DATE', 'PUBLISH_DATE', 'ROE_TTM', 'GROSS_INCOME_RATIO_TTM', 'NET_PROFIT', 'INCOME', 'MARKET_VALUE']]
    stock_finance = stock_finance[stock_finance['TICKER_SYMBOL'].isin(stock_industry['TICKER_SYMBOL'].unique().tolist())]
    stock_finance = stock_finance.sort_values('TICKER_SYMBOL')
    stock_finance = stock_finance.merge(stock_industry[['TICKER_SYMBOL', 'INDUSTRY_NAME']], on=['TICKER_SYMBOL'], how='left')
    stock_finance = stock_finance.dropna(subset=['TICKER_SYMBOL', 'END_DATE', 'PUBLISH_DATE', 'MARKET_VALUE', 'INDUSTRY_NAME'])

    income = stock_finance[['TICKER_SYMBOL', 'END_DATE', 'PUBLISH_DATE', 'INCOME', 'MARKET_VALUE', 'INDUSTRY_NAME']]
    income = income.dropna(subset=['INCOME'])
    income = income[['INDUSTRY_NAME', 'INCOME']].groupby('INDUSTRY_NAME').sum()

    net_profit = stock_finance[['TICKER_SYMBOL', 'END_DATE', 'PUBLISH_DATE', 'NET_PROFIT', 'MARKET_VALUE', 'INDUSTRY_NAME']]
    net_profit = net_profit.dropna(subset=['NET_PROFIT'])
    net_profit = net_profit[['INDUSTRY_NAME', 'NET_PROFIT']].groupby('INDUSTRY_NAME').sum()

    gross_ttm = stock_finance[['TICKER_SYMBOL', 'END_DATE', 'PUBLISH_DATE', 'GROSS_INCOME_RATIO_TTM', 'MARKET_VALUE', 'INDUSTRY_NAME']]
    gross_ttm = gross_ttm.dropna(subset=['GROSS_INCOME_RATIO_TTM'])
    gross_ttm = gross_ttm[(gross_ttm['GROSS_INCOME_RATIO_TTM'] <= gross_ttm['GROSS_INCOME_RATIO_TTM'].quantile(0.95)) & (gross_ttm['GROSS_INCOME_RATIO_TTM'] >= gross_ttm['GROSS_INCOME_RATIO_TTM'].quantile(0.05))]
    gross_ttm_mv = gross_ttm[['INDUSTRY_NAME', 'MARKET_VALUE']].groupby('INDUSTRY_NAME').sum().reset_index().rename(columns={'MARKET_VALUE': 'TOTAL_MARKET_VALUE'})
    gross_ttm = gross_ttm.merge(gross_ttm_mv, on=['INDUSTRY_NAME'], how='left')
    gross_ttm['GROSS_INCOME_RATIO_TTM'] = gross_ttm['GROSS_INCOME_RATIO_TTM'] * gross_ttm['MARKET_VALUE'] / gross_ttm['TOTAL_MARKET_VALUE']
    gross_ttm = gross_ttm[['INDUSTRY_NAME', 'GROSS_INCOME_RATIO_TTM']].groupby('INDUSTRY_NAME').sum()

    roe_ttm = stock_finance[['TICKER_SYMBOL', 'END_DATE', 'PUBLISH_DATE', 'ROE_TTM', 'MARKET_VALUE', 'INDUSTRY_NAME']]
    roe_ttm = roe_ttm.dropna(subset=['ROE_TTM'])
    roe_ttm = roe_ttm[(roe_ttm['ROE_TTM'] <= roe_ttm['ROE_TTM'].quantile(0.95)) & (roe_ttm['ROE_TTM'] >= roe_ttm['ROE_TTM'].quantile(0.05))]
    roe_ttm_mv = roe_ttm[['INDUSTRY_NAME', 'MARKET_VALUE']].groupby('INDUSTRY_NAME').sum().reset_index().rename(columns={'MARKET_VALUE': 'TOTAL_MARKET_VALUE'})
    roe_ttm = roe_ttm.merge(roe_ttm_mv, on=['INDUSTRY_NAME'], how='left')
    roe_ttm['ROE_TTM'] = roe_ttm['ROE_TTM'] * roe_ttm['MARKET_VALUE'] / roe_ttm['TOTAL_MARKET_VALUE']
    roe_ttm = roe_ttm[['INDUSTRY_NAME', 'ROE_TTM']].groupby('INDUSTRY_NAME').sum()

    industry_finance = income.merge(net_profit, left_index=True, right_index=True, how='outer').merge(gross_ttm, left_index=True, right_index=True, how='outer').merge(roe_ttm, left_index=True, right_index=True, how='outer')
    return industry_finance

def get_industry_tracking(report_date, sw_type):
    stock_info = HBDB().read_stock_info()
    stock_info.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/stock_info.hdf', key='table', mode='w')
    stock_info = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/stock_info.hdf', key='table')
    stock_info = stock_info.rename(columns={'zqdm': 'TICKER_SYMBOL', 'zqmc': 'SEC_SHORT_NAME', 'ssrq': 'ESTABLISH_DATE'})
    stock_info['ESTABLISH_DATE'] = stock_info['ESTABLISH_DATE'].astype(int).astype(str)
    stock_info = stock_info.loc[stock_info['TICKER_SYMBOL'].str.len() == 6]
    stock_info = stock_info.loc[stock_info['TICKER_SYMBOL'].astype(str).str.slice(0, 1).isin(['0', '3', '6'])]
    stock_info = stock_info[stock_info['ESTABLISH_DATE'] <= (datetime.strptime(report_date, '%Y%m%d') - timedelta(365)).strftime('%Y%m%d')]

    stock_industry = HBDB().read_stock_industry()
    stock_industry.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/stock_industry.hdf', key='table', mode='w')
    stock_industry = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/stock_industry.hdf', key='table')
    stock_industry = stock_industry.rename(columns={'zqdm': 'TICKER_SYMBOL', 'fldm': 'INDUSTRY_ID', 'flmc': 'INDUSTRY_NAME', 'hyhfbz': 'INDUSTRY_VERSION', 'fljb': 'INDUSTRY_TYPE', 'qsrq': 'BEGIN_DATE', 'jsrq': 'END_DATE', 'sfyx': 'IS_NEW', 'credt_etl': 'CREDT_ETL'})
    stock_industry['IS_NEW'] = stock_industry['IS_NEW'].astype(int)
    stock_industry['INDUSTRY_VERSION'] = stock_industry['INDUSTRY_VERSION'].astype(int)
    stock_industry['INDUSTRY_TYPE'] = stock_industry['INDUSTRY_TYPE'].astype(int)
    stock_industry = stock_industry[stock_industry['IS_NEW'] == 1]
    stock_industry = stock_industry[stock_industry['INDUSTRY_VERSION'] == 2]
    stock_industry = stock_industry[stock_industry['INDUSTRY_TYPE'] == sw_type]
    if sw_type == 1:
        stock_industry = stock_industry[~stock_industry['INDUSTRY_NAME'].isin(['综合', '社会服务'])]
    if sw_type == 3:
        stock_industry = stock_industry[stock_industry['INDUSTRY_NAME'].isin(['钴', '锂', '铝', '锂电池', '电池化学品', '硅料硅片', '光伏电池组件', '逆变器', '航空装备', '军工电子', '电动乘用车', '车身附件及饰件', '底盘与发动机系统', '空调', '冰洗', '酒店', '白酒', '数字芯片设计', '模拟芯片设计', '半导体设备', '医疗研发外包'])]
    stock_industry = stock_industry[stock_industry['TICKER_SYMBOL'].isin(stock_info['TICKER_SYMBOL'].unique().tolist())]

    last_report_date = '{0}{1}'.format(str(int(report_date[:4]) - 1), report_date[4:])
    industry_finance = get_industry_index(report_date, stock_industry)
    industry_finance_cols = list(industry_finance.columns)
    industry_finance.columns = [col + '_' + report_date for col in industry_finance_cols]
    last_industry_finance = get_industry_index(last_report_date, stock_industry)
    last_industry_finance_cols = list(last_industry_finance.columns)
    last_industry_finance.columns = [col + '_' + last_report_date for col in last_industry_finance_cols]
    industry_finance = pd.concat([industry_finance, last_industry_finance], axis=1)
    for col in ['INCOME', 'NET_PROFIT']:
        industry_finance[col + '_YOY_' + report_date] = (industry_finance[col + '_' + report_date] - industry_finance[col + '_' + last_report_date]) / abs(industry_finance[col + '_' + last_report_date]) * 100.0
    for col in ['GROSS_INCOME_RATIO_TTM', 'ROE_TTM']:
        industry_finance[col + '_YOY_' + report_date] = industry_finance[col + '_' + report_date] - industry_finance[col + '_' + last_report_date]
    return industry_finance

def get_result(report_date_list, sw_type_list):
    for sw_type in sw_type_list:
        industry_finance_list = []
        for report_date in report_date_list:
            industry_finance_list.append(get_industry_tracking(report_date, sw_type))
        industry_finance_all = pd.concat(industry_finance_list, axis=1)

        index_cn_dic = {'INCOME': '营业收入', 'NET_PROFIT': '归母净利润', 'GROSS_INCOME_RATIO_TTM': '毛利率（TTM）', 'ROE_TTM': 'ROE（TTM）'}
        fig_size_dic = {1: (12, 6), 2: (20, 8), 3: (12, 6)}

        for index in ['INCOME', 'NET_PROFIT', 'GROSS_INCOME_RATIO_TTM', 'ROE_TTM']:
            for report_date in report_date_list:
                data = industry_finance_all[[index + '_YOY_' + report_date]]
                data = data.sort_values(index + '_YOY_' + report_date, ascending=False)

                fig, ax = plt.subplots(figsize=fig_size_dic[sw_type])
                ax.bar(range(len(data.index)), data[index + '_YOY_' + report_date], color='#C94649', tick_label=data.index, label=index_cn_dic[index] + '_' + report_date)
                plt.legend(loc=1)
                plt.xticks(rotation=90)
                plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
                if index in ['INCOME', 'NET_PROFIT']:
                    plt.ylabel('同比增速')
                if index in ['GROSS_INCOME_RATIO_TTM', 'ROE_TTM']:
                    plt.ylabel('同比绝对变化')
                plt.tight_layout()
                plt.savefig('{0}{1}{2}'.format('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/', 'sw{0}/'.format(str(sw_type)), index_cn_dic[index] + '_' + report_date))

        report_date_disp_list = [report_date_list[len(report_date_list) - i - 1] for i in range(len(report_date_list))]
        for index in ['GROSS_INCOME_RATIO_TTM', 'ROE_TTM']:
            data_list = []
            for report_date in report_date_disp_list:
                data = industry_finance_all[[index + '_' + report_date]].rename(columns={index + '_' + report_date: index}).reset_index()
                data['REPORT_DATE'] = report_date
                data = data.sort_values(index, ascending=False)
                data_list.append(data)
            data = pd.concat(data_list)

            fig, ax = plt.subplots(figsize=fig_size_dic[sw_type])
            sns.barplot(x='INDUSTRY_NAME', y=index, data=data, hue='REPORT_DATE', hue_order=report_date_disp_list, palette=['#C94649', '#8588B7'])
            plt.legend(loc=1)
            plt.xticks(rotation=90)
            plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
            plt.xlabel('')
            plt.ylabel(index_cn_dic[index])
            plt.tight_layout()
            plt.savefig('{0}{1}{2}'.format('D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/', 'sw{0}/'.format(str(sw_type)), index_cn_dic[index]))
    return


if __name__ == '__main__':
    report_date_list = ['20211231', '20220331']
    sw_type_list = [1, 2, 3]
    get_result(report_date_list, sw_type_list)




# -*- coding: utf-8 -*-

import pandas as pd
import pymysql
import hbshare as hbs
import warnings
warnings.filterwarnings('ignore', category=pymysql.Warning)


class HBDB:
    def __init__(self):
        pass

    def get_df(self, sql, db, page_size=2000):
        data = hbs.db_data_query(db, sql, page_size=page_size, timeout=120)
        pages = data['pages']
        data = pd.DataFrame(data['data'])
        if pages > 1:
            for page in range(2, pages + 1):
                temp_data = hbs.db_data_query(db, sql, page_size=page_size, page_num=page, timeout=120)
                data = pd.concat([data, pd.DataFrame(temp_data['data'])], axis=0)
        return data

    def read_report_date(self):
        sql = "select distinct(jsrq) as REPORT_DATE FROM st_fund.t_st_gm_gpzh"
        df = self.get_df(sql, db='funduser')
        return df

    def read_cal(self, start, end):
        sql = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ>={0} and JYRQ<={1}".format(start, end)
        df = self.get_df(sql, db='readonly')
        return df

    def read_fund_info(self):
        sql = "select jjdm, jjmc, jjjc, clrq, zzrq, cpfl, kffb, jjfl, ejfl from st_fund.t_st_gm_jjxx where cpfl='2'"
        df = self.get_df(sql, db='funduser')
        return df

    def read_stock_fund_info(self):
        sql = "select jjdm, jjmc, jjjc, clrq, zzrq, ejfl, kffb from st_fund.t_st_gm_jjxx where jjzt='0' and cpfl='2' and ejfl in ('13', '35', '37')"
        df = self.get_df(sql, db='funduser')
        return df

    def read_company_fund_given_code(self, code):
        sql = "select JGDM, JGMC, JJDM from funddb.JJGL where JGDM='{0}'".format(code)
        df = self.get_df(sql, db='readonly')
        return df

    def read_fund_company_given_code(self, code):
        sql = "select JJDM, JGDM, JGMC from funddb.JJGL where JJDM='{0}'".format(code)
        df = self.get_df(sql, db='readonly')
        return df

    def read_fund_manager_given_code(self, code):
        sql = "select jjdm, rydm, ryxm, ryzw, bdrq, ryzt, rzrq, lrrq, lryy from st_fund.t_st_gm_jjjl where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_given_manager_code(self,manager_code):
        sql = "select jjdm, rydm, ryxm, ryzw, bdrq, ryzt, rzrq, lrrq, lryy from st_fund.t_st_gm_jjjl where rydm='{0}'".format(manager_code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_target_given_code(self, code):
        sql = "select jjdm, tzmb from funddb.JJXX2 where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='readonly')
        return df

    def read_fund_scale_given_code(self, code):
        sql = "select jjdm, bblb1, jsrq, ggrq, zcjz from funddb.CWZB where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='readonly')
        return df

    def read_fund_scale_given_date(self, date):
        sql = "select jjdm, bblb1, jsrq, ggrq, zcjz from funddb.CWZB where jsrq='{0}'".format(date)
        df = self.get_df(sql, db='readonly')
        return df

    def read_fund_share_given_code(self, code):
        sql = "select jjdm, bblb, jsrq, plrq, qsrq, qcfe, qjsgfe, cfzjfe, wjshfe, qmfe from st_fund.t_st_gm_febd where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_holder_given_code(self, code):
        sql = "select jjdm, jsrq, ggrq, jgcyfe, jgcybl, grcyfe, grcybl from st_fund.t_st_gm_fecyrbd where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_manager_product_given_code(self, manager_code):
        sql = "select rydm, jsrq, zgcpsl, zgcpgm, dbjj from st_fund.t_st_gm_jlzgcptj where rydm='{0}'".format(manager_code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_manager_achievement_given_code(self, manager_code, code):
        sql = "select rydm, jjdm, qsrq, jsrq, rqts, hbrq, rqnhhb, zdyl, zdhc, nhxpbl, kmbl from st_fund.t_st_gm_jlrqhb where rydm='{0}' and jjdm='{1}'".format(manager_code, code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_position_given_code(self, code):
        sql = "select jjdm, jsrq, jjzzc, gptzsz, zqzsz, jjtzszhj, hbzjsz from st_fund.t_st_gm_zcpz where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_position_given_date(self, date):
        sql = "select jjdm, jsrq, jjzzc, gptzsz, zqzsz, jjtzszhj, hbzjsz from st_fund.t_st_gm_zcpz where jsrq='{0}'".format(date)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_cr_given_code(self, code):
        sql = "select jjdm, jsrq, qsdzb, qsdzbtlpj, qwdzb, qwdzbtlpj from st_fund.t_st_gm_cgjzd where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_turnover_given_code(self, code):
        sql = "select jjdm, jsrq, tjqj, hsl from st_fund.t_st_gm_jjhsl where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_nav_given_code(self, code):
        sql = "select jjdm, jzrq, jjjz, ljjz from st_fund.t_st_gm_jjjz where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_benchmark_nav_given_code(self, code):
        sql = "select jjdm, jzrq, jzhbdr from st_fund.t_st_gm_jzrhb where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_benchmark_given_code(self, code):
        sql = "select JJDM, BDRQ, BJJZ, SYZT from funddb.BJJZ where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='readonly')
        return df

    def read_fund_wind_size_given_code(self, code):
        sql = "select jjdm, wdszsx from st_fund.t_st_gm_jjxx where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_return_given_code_and_date(self, code, date):
        sql = "select jjdm, jzrq, zblb, rqnp, zbnp, zbnhnp from st_fund.t_st_gm_rqjhb where jjdm='{0}' and jzrq >= '{1}'".format(code, date)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_samekind_return_given_code_and_date(self, code, date):
        sql = "select jjdm, jzrq, zblb, pmnpyj, slnpyj, zbnpyj, pmnpej, slnpej, zbnpej, zbnhnpyj, zbnhnpej from st_fund.t_st_gm_rqjhbpm where jjdm='{0}' and jzrq >= '{1}'".format(code, date)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_year_return_given_code(self, code):
        sql = "select a.jjdm, a.tjnf, a.hb1n, b.hb1npmyj, b.hb1npmej, b.hb1npmslyj, b.hb1npmslej, b.pjhb1nyj, b.pjhb1nej from st_fund.t_st_gm_nhb a left join st_fund.t_st_gm_nhbpm b on a.jjdm=b.jjdm and a.tjnf=b.tjnf where a.jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_year_volatility_given_code(self, code):
        sql = "select a.jjdm, a.tjnf, a.zzbnp1n, a.znhzbnp1n, b.zbdpm1nyj, b.tlzbdsl1nyj, b.zbdpm1nej, b.tlzbdsl1nej, b.pjzbd1nyj, b.pjnhzbd1nyj, b.pjzbd1nej, b.pjnhzbd1nej from st_fund.t_st_gm_nbdl a left join st_fund.t_st_gm_nbdlpm b on a.jjdm=b.jjdm and a.tjnf=b.tjnf where a.jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_year_sharpratio_given_code(self, code):
        sql = "select a.jjdm, a.tjnf, a.zzbnp1n, a.znhzbnp1n, b.zxppm1nyj, b.tlzxpsl1nyj, b.zxppm1nej, b.tlzxpsl1nej, b.pjzxp1nyj, b.pjznhxp1nyj, b.pjzxp1nej, b.pjznhxp1nej from st_fund.t_st_gm_nxpbl a left join st_fund.t_st_gm_nxpblpm b on a.jjdm=b.jjdm and a.tjnf=b.tjnf where a.jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_year_maxdrawdown_given_code(self, code):
        sql = "select a.jjdm, a.tjnf, a.zbnp1n, a.zdhchfts1n, a.zdhccxts1n, b.hczdpm1nyj, b.tlhczdsl1nyj, b.hczdpm1nej, b.tlhczdsl1nej, b.pjhczd1nyj, b.pjhczd1nej from st_fund.t_st_gm_nzdhc a left join st_fund.t_st_gm_nzdhcpm b on a.jjdm=b.jjdm and a.tjnf=b.tjnf where a.jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_brinson_attribution_given_code(self, code):
        sql = "select jjdm, tjrq, portfolio_return, benchmark_return, asset_allo, sector_allo, equity_selection, trading, portfolio_weight, benchmark_weight from st_fund.r_st_hold_excess_attr_df where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_barra_attribution_given_code(self, code):
        sql = "select jjdm, tjrq, attr_type, style_factor, data_type, data_value from st_fund.r_st_nav_attr_df where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_gptzzjb_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, gptzzjb from st_fund.t_st_gm_zcpz where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_gptzzjb_given_date(self, date):
        sql = "select jjdm, jsrq, gptzzjb from st_fund.t_st_gm_zcpz where jsrq='{0}'".format(date)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_holding_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zqdm, zqmc, ccsz, ccsl, zjbl from st_fund.t_st_gm_gpzh where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_fund_holding_diff_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zqdm, zqmc, zclb, zgblbd, sfsqzcg from st_fund.t_st_gm_jjcgbd where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_fund_holding_semi_diff_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zqdm, zqmc, zzjbl, zjblpm, ggzjzbjbd from st_fund.t_st_gm_gpzhggtj where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_fund_valuation_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zclb, pe, pb, roe, dividend from st_fund.t_st_gm_jjggfg where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=2000)
        return df

    def read_stock_market_value_given_date(self, date):
        sql = "select b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.TotalMV AS MARKET_VALUE from hsjy_gg.LC_DIndicesForValuation a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    def read_star_stock_market_value_given_date(self, date):
        sql = "select b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.TotalMV AS MARKET_VALUE from hsjy_gg.LC_STIBDIndiForValue a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    def read_stock_valuation_given_date(self, date):
        sql = "select b.SECUCODE AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.TotalMV AS MARKET_VALUE, a.PE AS PE_TTM, a.PB AS PB_LF, a.PEG, a.DividendRatio AS DIVIDEND_RATIO_TTM, a.ForwardPEHR AS FORWARD_PEHR from hsjy_gg.LC_DIndicesForValuation a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly')
        return df

    def read_star_stock_valuation_given_date(self, date):
        sql = "select b.SECUCODE AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.TotalMV AS MARKET_VALUE, a.PETTM AS PE_TTM, a.PB AS PB_LF, a.PEG, a.DividendRatioTTM AS DIVIDEND_RATIO_TTM, a.ForwardPEHR AS FORWARD_PEHR from hsjy_gg.LC_STIBDIndiForValue a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly')
        return df

    def read_stock_finance_given_date(self, date):
        sql = "select b.SECUCODE AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.InfoPublDate, 'yyyyMMdd') AS PUBLISH_DATE, to_char(a.EndDate, 'yyyyMMdd') AS END_DATE, a.NetProfit AS NET_PROFIT, a.MainIncomePS AS MAIN_INCOME_PS, a.ROETTM AS ROE_TTM, a.GrossIncomeRatioTTM AS GROSS_INCOME_RATIO_TTM, a.NetProfitRatioTTM AS NET_PROFIT_RATIO_TTM, a.EPSTTM AS EPS_TTM, a.OperCashFlowPSTTM AS OPER_CASH_FLOW_PS_TTM, a.NetAssetPS AS NET_ASSET_PS from hsjy_gg.LC_MainIndexNew a left join hsjy_gg.SecuMain b on a.CompanyCode=b.CompanyCode where to_char(a.EndDate, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly')
        return df

    def read_star_stock_finance_given_date(self, date):
        sql = "select b.SECUCODE AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.InfoPublDate, 'yyyyMMdd') AS PUBLISH_DATE, to_char(a.EndDate, 'yyyyMMdd') AS END_DATE, a.NPParentCompanyOwners AS NET_PROFIT, a.MainIncomePS AS MAIN_INCOME_PS, a.ROETTM AS ROE_TTM, a.GrossIncomeRatioTTM AS GROSS_INCOME_RATIO_TTM, a.NetProfitRatioTTM AS NET_PROFIT_RATIO_TTM, a.EPSTTM AS EPS_TTM, a.OperCashFlowPSTTM AS OPER_CASH_FLOW_PS_TTM, a.NetAssetPS AS NET_ASSET_PS from hsjy_gg.LC_STIBMainIndex a left join hsjy_gg.SecuMain b on a.CompanyCode=b.CompanyCode where to_char(a.EndDate, 'yyyyMMdd')='{0}' and a.IfMerged=1 and a.IfAdjusted=2".format(date)
        df = self.get_df(sql, db='readonly')
        return df

    def read_stock_daily_k_given_date(self, date):
        sql = "select b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.ClosePrice AS CLOSE_PRICE from hsjy_gg.QT_PerformanceData a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    def read_star_stock_daily_k_given_date(self, date):
        sql = "select b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.ClosePrice AS CLOSE_PRICE from hsjy_gg.LC_STIBPerformanceData a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    def read_stock_daily_k_given_code(self, code):
        sql = "select b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.ClosePrice AS CLOSE_PRICE from hsjy_gg.QT_PerformanceData a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where b.SecuCode='{0}'".format(code)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    def read_star_stock_daily_k_given_code(self, code):
        sql = "select b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.ClosePrice AS CLOSE_PRICE from hsjy_gg.LC_STIBPerformanceData a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where b.SecuCode='{0}'".format(code)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    # def read_stock_industry(self):
    #     sql = "select gsdm, xxfbrq, xxly, xyhfbz, ssxy, sfzx, qxrq, yjxydm, yjxymc, ejxydm, ejxymc from st_ashare.t_st_ag_gshyhfb"
    #     df = self.get_df(sql, db='alluser')
    #     return df

    def read_stock_industry(self):
        sql = "select zqdm, flmc, fldm, fljb, hyhfbz, qsrq, jsrq, sfyx, credt_etl from st_fund.t_st_gm_zqhyflb"
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_industry_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zclb, hyhfbz, fldm, flmc, zzjbl, hyzjzbjbd, hyzjzbltlpj, zgpbl, hyzgpbjbd, hyzgpbltlpj, zltgbl, hyzltgbjbd, hyzltgbltlpj from st_fund.t_st_gm_jjhyzhyszb where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_industry_theme(self):
        sql = "select hybh, hyzt, fldm, qsrq, jsrq from st_fund.t_st_gm_hyztpzb"
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_theme_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zblx, zclb, zgpszb, jsqbd, tlpj from st_fund.t_st_gm_ztcgbd where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_industry_info(self):
        sql = "select hyhfbz, fldm, flmc, zsdm, qsrq, jsrq, fljb, sfyx from st_market.t_st_zs_hyzsdmdyb"
        df = self.get_df(sql, db='alluser')
        return df

    def read_index_info(self):
        sql = "select a.IndexCode, b.SecuCode, b.ChiName, b.ChiNameAbbr from hsjy_gg.LC_IndexBasicInfo a left join hsjy_gg.SecuMain b on a.IndexCode=b.InnerCode"
        df = self.get_df(sql, db='readonly')
        return df

    def read_index_cons(self, index):
        sql = "select a.IndexCode, b.SecuCode, a.InfoSource, a.EndDate, a.Weight from hsjy_gg.LC_IndexComponentsWeight a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where a.IndexCode='{0}'".format(index)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    def read_fund_style(self):
        sql = "select fund_id, trade_date, type, cap_score, vcg_score, category from st_fund.r_st_gm_equity_score"
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_stock_style(self):
        sql = "select ticker, trade_date, type, cap_score, vcg_score, category from st_fund.r_st_gp_equity_score"
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_barra_style_exposure_given_dates(self, dates):
        dates = '"' + '","'.join(dates) + '"'
        sql = "select ticker, trade_date, size, beta, momentum, resvol, btop, sizenl, liquidity, earnyield, growth, leverage from st_ashare.r_st_barra_style_factor where trade_date in ({0})".format(dates)
        df = self.get_df(sql, db='alluser', page_size=200000)
        return df

    def read_index_daily_k_given_date_and_indexs(self, date, indexs):
        indexs = '"' + '","'.join(indexs) + '"'
        sql = "select scdm, zqdm, zqmc, jyrq, qspj, kpjg, spjg, zgjg, zdjg, zdsl, zdfd, bdfd, cjsl, cjjs, cjbs, ltsz, zsz from st_market.t_st_zs_hqql where jyrq>='{0}' and zqdm in ({1}) ".format(date, indexs)
        df = self.get_df(sql, db='alluser', page_size=200000)
        return df

    def read_industry_index_given_date_and_names(self, date, names):
        names = '"' + '","'.join(names) + '"'
        sql = "select zqmc, zqdm from st_market.t_st_zs_hqql where jyrq='{0}' and zqmc in ({1}) and left(zqdm, 1)='8' and length(zqdm)=6".format(date, names)
        df = self.get_df(sql, db='alluser', page_size=200000)
        return df

    def read_index_valuation_given_date_and_indexs(self, date, indexs):
        indexs = '"' + '","'.join(indexs) + '"'
        sql = "select scdm, zqdm, zqmc, jyrq, pe, pb, roe, gxl from st_market.t_st_zs_hqql where jyrq>='{0}' and zqdm in ({1}) ".format(date, indexs)
        df = self.get_df(sql, db='alluser', page_size=200000)
        return df

    def read_fund_nav_given_date_and_codes(self, date, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jzrq, jjjz, ljjz from st_fund.t_st_gm_jjjz where jzrq>='{0}' and jjdm in ({1}) ".format(date, codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_fund_nav_adj_given_date_and_codes(self, date, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jzrq, fqdwjz, hbfh, hbcl, hbdr from st_fund.t_st_gm_rhb where jzrq>='{0}' and jjdm in ({1}) ".format(date, codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_finance_data_given_date(self, date):
        sql = "select b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.InfoPublDate, 'yyyyMMdd') AS PUBLISH_DATE, to_char(a.EndDate, 'yyyyMMdd') AS END_DATE, ROETTM AS ROE_TTM, ROATTM AS ROA_TTM, ROICTTM AS ROIC_TTM, NetProfitRatioTTM AS NET_PROFIT_RATIO_TTM, GrossIncomeRatioTTM AS GROSS_INCOME_RATIO_TTM, EBITToToAssetsTTM AS EBIT_ASSET_RATIO_TTM, a.NetProfit AS NET_PROFIT, a.MainIncomePS AS INCOME_PS from hsjy_gg.LC_MainIndexNew a left join hsjy_gg.SecuMain b on a.CompanyCode=b.CompanyCode where to_char(a.EndDate, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    def read_bond_fund_info(self):
        sql = "select jjdm, jjmc, jjjc, clrq, zzrq, ejfl, kffb from st_fund.t_st_gm_jjxx where cpfl = '2' and jjzt not in ('3', 'c') and m_opt_type <> '03' and ejfl in ('21', '22', '23', '24', '25', '27', '28', '34', '35', '38')"
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_private_bond_fund_info(self):
        sql = "select jjdm, jjmc, jjjc, clrq, zzrq, jjfl from st_hedge.t_st_jjxx where cpfl = '4' and jjzt not in ('3') and m_opt_type <> '03' and jjfl in ('2')"
        df = self.get_df(sql, db='highuser', page_size=200000)
        return df

    def read_bond_fund_position_given_code(self, code):
        sql = "select jjdm, jsrq, gptzsz, kzzsz from st_fund.t_st_gm_zcpz where jjdm='{0}'".format(code)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_cumret_given_code(self, code, start, end):
        sql = "SELECT a.jjdm AS FUND_CODE, b.jzrq AS TRADE_DATE, b.hbcl AS CUM_RET FROM st_fund.t_st_gm_jjxx a, st_fund.t_st_gm_rhb b WHERE a.cpfl = '2' AND a.jjdm = b.jjdm AND a.jjzt not in ('3', 'c') AND a.m_opt_type <> '03' AND a.jjdm = '{0}' AND b.jzrq >= {1} AND b.jzrq <= {2} order by b.jzrq".format(code, start, end)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_private_fund_cumret_given_code(self, code, start, end):
        sql = "SELECT a.jjdm AS FUND_CODE, b.jzrq AS TRADE_DATE, b.fqdwjz AS ADJ_NAV FROM st_hedge.t_st_jjxx a, st_hedge.t_st_rhb b WHERE a.cpfl = '4' AND a.jjdm = b.jjdm AND a.jjzt not in ('3') AND a.m_opt_type <> '03' AND a.jjdm = '{0}' AND b.jzrq >= {1} AND b.jzrq <= {2} order by b.jzrq".format(code, start, end)
        df = self.get_df(sql, db='highuser', page_size=200000)
        return df

    def read_fund_cumret_given_date(self, date):
        sql = "SELECT jjdm, jzrq, hbcl FROM funddb.jjhb WHERE jzrq = '{0}'".format(date)
        df = self.get_df(sql, db='readonly')
        return df

    def read_stock_info(self):
        sql = "SELECT zqdm, zqjc, ssrq FROM st_ashare.t_st_ag_zqzb where zqlb=1 and sszt=1"
        df = self.get_df(sql, db='alluser', page_size=200000)
        return df

    def read_consensus_given_date(self, date, type):
        sql = "SELECT S_INFO_WINDCODE AS TICKER_SYMBOL, EST_DT, ROLLING_TYPE, EST_PE, EST_PB, EST_PEG, EST_DPS, NET_PROFIT AS EST_NET_PROFIT, EST_OPER_REVENUE, EST_ROE, EST_EPS, EST_CFPS, EST_BPS FROM wind.AShareConsensusRollingData where EST_DT='{0}' AND ROLLING_TYPE='{1}'".format(date, type)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    def read_stock_heightest_given_date(self, date):
        sql = "select b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.ClosePrice AS CLOSE_PRICE, a.HighestPrice, a.HighestPriceRW, a.HighestPriceTW, a.HighestPriceRM, a.HighestPriceTM, a.HighestPriceR3M AS HighestPriceRMThree, a.HighestPriceR6M AS HighestPriceRMSix, a.HighestPriceR12M AS HighestPriceRY, a.HighestPriceYTD from hsjy_gg.QT_PerformanceData a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    def read_star_stock_heightest_given_date(self, date):
        sql = "select b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.ClosePrice AS CLOSE_PRICE, a.HighestPrice, a.HighestPriceRW, a.HighestPriceTW, a.HighestPriceRM, a.HighestPriceTM, a.HighestPriceRMThree, a.HighestPriceRMSix, a.HighestPriceRY, a.HighestPriceYTD from hsjy_gg.LC_STIBPerformanceData a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df


if __name__ == '__main__':
    dates = HBDB().read_fund_barra_attribution_given_code('450004')
    barra_style_exposure = HBDB().read_barra_style_exposure_given_dates(fund_zc_wmv['RECENT_TRADE_DATE'].unique().tolist())
    barra_style_exposure.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/barra_style_exposure.hdf', key='table', mode='w')


# -*- coding: utf-8 -*-

import logging


class Logger:
    def __init__(self,
                 clevel=logging.DEBUG,
                 flevel=logging.DEBUG,
                 name='application',
                 path=None):
        self.logger = logging.getLogger(name)
        self.logger.setLevel(logging.DEBUG)
        fmt = logging.Formatter('[%(asctime)s] [%(levelname)s] %(message)s',
                                '%Y-%m-%d %H:%M:%S')
        sh = logging.StreamHandler()
        sh.setFormatter(fmt)
        sh.setLevel(clevel)
        self.logger.addHandler(sh)
        if path is not None:
            fh = logging.FileHandler(path)
            fh.setFormatter(fmt)
            fh.setLevel(flevel)
            self.logger.addHandler(fh)

    def debug(self, message):
        self.logger.debug(message)

    def info(self, message):
        self.logger.info(message)

    def warning(self, message):
        self.logger.warning(message)

    def error(self, message):
        self.logger.error(message)

    def critical(self, message):
        self.logger.critical(message)

    def exception(self, message):
        self.logger.exception(message)


logger = Logger()



"""
CTA基金的净值归因模块
结构：
1、整体区间的归因结果；
2、滚动月度的归因结果：暴露+收益的时序图；
"""
import pandas as pd
from sqlalchemy import create_engine
from hbshare.fe.common.util.verifier import verify_type
from hbshare.fe.common.util.data_loader import CtaNavAttributionLoader
from hbshare.fe.common.util.regressions import Regression
# from hbshare.rm_associated.util.config import cta_factors, cta_factor_map_dict
# from hbshare.rm_associated.util.plot_util import draw_summary_bar_style, draw_area_bar, draw_heatmap


class CtaNavAttribution:
    def __init__(self, fund_id, start_date, end_date, factor_dict, table_name="cta_index"):
        self.fund_id = fund_id
        self.start_date = start_date
        self.end_date = end_date
        self.factor_dict = factor_dict
        self.table_name = table_name
        self._verify_input_param()
        self._load_data()

    def _verify_input_param(self):
        verify_type(self.fund_id, 'fund_id', str)
        verify_type(self.start_date, 'start_date', str)
        verify_type(self.end_date, 'end_date', str)
        verify_type(self.factor_dict, 'factor_dict', dict)
        verify_type(self.table_name, 'table_name', str)

    def _load_data(self):
        data = CtaNavAttributionLoader(
            self.fund_id, self.start_date, self.end_date, self.factor_dict, self.table_name).load()
        self.trading_day_list = data['trading_day_list']
        self.fund_nav_series = data['fund_nav_series']
        self.factor_data = data['factor_data']

    @staticmethod
    def _process_data(df):
        factor_df = df.copy()
        factor_df['rev_short'] = -1 * factor_df['tsmom_d1']
        factor_df['tendency_medium'] = factor_df[['tsmom_d2', 'tsmom_d3', 'tsmom_d5']].mean(axis=1)
        # factor_df['tendency_long'] = factor_df[['tsmom_d10', 'tsmom_d20', 'tsmom_d30']].mean(axis=1)
        factor_df['tendency_long'] = factor_df[['tsmom_d10', 'tsmom_d20']].mean(axis=1)
        factor_df['roll_return'] = factor_df['carry_d20_q70']
        factor_df['basis_mom_short'] = factor_df['tsbasismom_d5']
        factor_df['basis_mom_long'] = factor_df['tsbasismom_d20']

        # factor_df = factor_df[['tsmom_d5', 'tsmom_d20', 'roll_return', 'basis_mom_5', 'basis_mom_20']]
        # factor_df = factor_df[['tendency_medium', 'tendency_long', 'roll_return', 'basis_mom_5', 'basis_mom_20']]
        # factor_df = factor_df[['tsmom_d1', 'tsmom_d5', 'tsmom_d20', 'roll_return', 'basis_mom_5', 'basis_mom_20']]
        factor_df = factor_df[
            ['rev_short', 'tendency_medium', 'tendency_long', 'roll_return', 'basis_mom_short', 'basis_mom_long']]

        return factor_df

    def _calculate_period_attr(self, return_series, factor_df, start_date, end_date):
        lower, upper = None, None
        strategy_method, attribution_effort = 'ols', 'weak'

        period_return_series = return_series[(return_series.index > start_date) & (return_series.index <= end_date)]
        period_factor_df = factor_df[(factor_df.index > start_date) & (factor_df.index <= end_date)]

        sr_obj = Regression(period_factor_df, period_return_series, upper=upper, lower=lower,
                            method=strategy_method, effort=attribution_effort)
        solve_dict = sr_obj.solve()
        factor_beta_series = solve_dict['solution'].reindex(self.factor_dict.keys())
        r_square = solve_dict['r_square']

        calc_df = factor_df[factor_beta_series.index.tolist()]
        factor_cum_return = calc_df.fillna(0.).add(1.0).prod().subtract(1.0)
        perf_attr = factor_beta_series.multiply(factor_cum_return)
        perf_attr = pd.merge(factor_beta_series.to_frame('factor_exposure'), perf_attr.to_frame('return_attr'),
                             left_index=True, right_index=True)

        return perf_attr, r_square

    def calculate(self):
        idx = self.fund_nav_series.index.intersection(self.factor_data.index)
        return_series = self.fund_nav_series.reindex(idx).sort_index().pct_change().dropna()
        factor_df = self.factor_data.reindex(idx).sort_index().pct_change().dropna()
        # factor_df = self._process_data(factor_df)

        # overall attr
        perf_attr, r_square = self._calculate_period_attr(
            return_series, factor_df, self.start_date, self.end_date)
        perf_attr.index.name = 'style_factor'
        attr_summary = perf_attr.reset_index()

        # rolling attr
        exposure_list = []
        return_attr_list = []
        ac_date_list = []
        r_square_list = []
        date_list = [x for x in self.trading_day_list if x >= self.start_date]
        for date in date_list:
            period_start = self.trading_day_list[self.trading_day_list.index(date) - 6]
            period_nav_series = self.fund_nav_series[
                (self.fund_nav_series.index > period_start) & (self.fund_nav_series.index <= date)]
            if len(period_nav_series) <= 20:
                continue
            perf_attr, r_square = self._calculate_period_attr(
                return_series, factor_df, period_start, date)
            ac_date_list.append(date)
            r_square_list.append(r_square)

            exposure_series = perf_attr['factor_exposure']
            exposure_series.name = date

            return_attr_series = perf_attr['return_attr']
            return_attr_series.name = date

            exposure_list.append(exposure_series)
            return_attr_list.append(return_attr_series)

        exposure_series_all = pd.concat(exposure_list, axis=1).T.sort_index()
        return_attr_series_all = pd.concat(return_attr_list, axis=1).T.sort_index()
        r_square_series = pd.Series(index=ac_date_list, data=r_square_list)

        output = {
            "attr_summary": attr_summary,
            "exposure_series": exposure_series_all,
            "return_attr_series": return_attr_series_all,
            "r_square_series": r_square_series
        }

        return output


def get_fund_list_from_mysql():
    sql_params = {
        "ip": "192.168.223.152",
        "user": "readonly",
        "pass": "c24mg2e6",
        "port": "3306",
        "database": "work"
    }
    engine_params = "mysql+pymysql://{}:{}@{}:{}/{}".format(sql_params['user'], sql_params['pass'],
                                                            sql_params['ip'],
                                                            sql_params['port'], sql_params['database'])

    sql_script = "SELECT * FROM fund_list where class = 'cta'"
    engine = create_engine(engine_params)
    fund_info = pd.read_sql(sql_script, engine)
    # fof
    fund_info = fund_info[fund_info['fof'] == 1]

    return fund_info


if __name__ == '__main__':
    # fund_df = get_fund_list_from_mysql()
    # fund_df = fund_df[['code', 'name', 'type']]
    # fund_df = fund_df[~fund_df['code'].isin(['SGZ853', 'SJJ884'])]
    #
    # for f_id in fund_df['code'].tolist():
    #     attr_res = CtaNavAttribution(f_id, '20191227', '20210903').get_construct_result()
    #     fund_df.loc[fund_df['code'] == f_id,
    #                 ['tendency_short', 'tendency_medium', 'tendency_long',
    #                  'roll_return', 'basis_mom_5', 'basis_mom_20', 'r_square']] = attr_res['exposure'].tolist()[1:]
    #
    # print(fund_df)

    f_dict = {
        'tsmom_d20': '中期时序动量',
        'tsmom_d250': '长期时序动量',
        'tsmom_d3': '短期时序动量',
        'xsmom_d60_q75': '中长期截面动量',
        'carry_d20_q70': '展期收益',
        'tsrev_d1': '短期反转',
        'tsbasismom_d3': '短期基差动量',
        'xssigma_d20_q75': '波动率',
        'xsmrchg_d1_q50': '会员持仓'
    }

    t_name = "cta_index"

    res = CtaNavAttribution('SNP957', '20201231', '20220527', f_dict, t_name).calculate()

    # factor_order = list(cta_factor_map_dict.keys())
    # attribution_df = res['attr_summary'].set_index('style_factor').reindex(factor_order)
    # attribution_df['factor_exposure'] = attribution_df['factor_exposure'].round(2)
    # attribution_df['return_attr'] = (attribution_df['return_attr'] * 100).round(2)
    # attribution_df.index = attribution_df.index.map(cta_factor_map_dict)
    # summary_bar = draw_summary_bar_style(attribution_df)
    # summary_bar.render("D:\\kevin\\123.html")
    #
    # exposure_series = res['exposure_series'].round(3)
    # exposure_series.columns = [cta_factor_map_dict[x] for x in exposure_series.columns]
    # exposure_heatmap = draw_heatmap(exposure_series, title="因子暴露时序分析", min_value=-1, max_value=1)
    # exposure_heatmap.render('D:\\kevin\\456.html')
    #
    # return_attr_series = (res['return_attr_series'][factor_order] * 100).round(2)
    # return_attr_series.columns = [cta_factor_map_dict[x] for x in return_attr_series.columns]
    # return_attr_stack_bar = draw_area_bar(return_attr_series, title="风格收益贡献时序分析")
    # return_attr_stack_bar.render('D:\\kevin\\789.html')


# -*- coding: utf-8 -*-

import json
import requests
import sys
import traceback
import time
import pandas as pd
from hbshare.fe.xwq.analysis.utils.const_var import TimeDateFormat
from hbshare.fe.xwq.analysis.utils.timedelta_utils import TimeDateUtil
from hbshare.base.upass import is_prod_env


class Curl_Request:
    def __init__(self):
        pass

    def curl_request(self, url, method, data=None, max_times=30):
        """
        处理请求的函数
        """
        valid = False
        times = 0
        dic = None
        headers = {
            'Content-Type': 'application/json;charset=UTF-8',
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'zh-cn',
            'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) '
                          'AppleWebKit/573.36 (KHTML, like Gecko) '
                          'Chrome/73.0.3683.86 Safari/573.36',
            'Authorization': 'Bearer '
        }
        while not valid and times < max_times:
            try:
                if method == 'GET':
                    r = requests.get(url, headers=headers)
                elif method == 'POST':
                    r = requests.post(url, data=json.dumps(data), headers=headers)
                elif method == 'POST JSON':
                    r = requests.post(url, json=data, headers=headers)
                else:
                    r = requests.put(url, json=data, headers=headers)
                dic = json.loads(r.text)
                valid = True
            except Exception as e:
                exc_type, exc_value, exc_traceback = sys.exc_info()
                print(repr(traceback.format_exception(exc_type, exc_value, exc_traceback)))
                valid = False
                times += 1
                time.sleep(5)
        return dic

    def get_df(self, url, method, data):
        df = self.curl_request(url, method, data)
        pages = df['body']['pages']
        df = pd.DataFrame(df['body']['records'])
        if pages > 1:
            for page in range(2, pages + 1):
                data['page'] = page
                temp_df = self.curl_request(url, method, data)
                df = pd.concat([df, pd.DataFrame(temp_df['body']['records'])], axis=0)
        return df

    def get_duration_between_dates(self, start, end):
        prod = 'ams.inner.howbuy.com'
        office = 'ams-data.intelnal.howbuy.com'
        url = 'http://%s/data/fund/sac/zsjq' % (prod if is_prod_env() else office)
        method = 'POST'
        data = {
            "tradingDay": {
                "startDate": "{0} 00:00:00".format(TimeDateUtil.convert_format(start, TimeDateFormat.YMD.value, TimeDateFormat.YMDHYPHEN.value)),
                "endDate": "{0} 00:00:00".format(TimeDateUtil.convert_format(end, TimeDateFormat.YMD.value, TimeDateFormat.YMDHYPHEN.value))
            },
            "fields": [
                "zqdm",
                "jyrq",
                "jq",
                "dataSource",
                "pages"
            ],
            "page": 1,
            "perPage": 2000
        }
        df = self.get_df(url, method, data)
        return df

if __name__ == '__main__':
    res = Curl_Request().get_duration_between_dates('20150101', '20220506')


#!/usr/bin/python
#coding:utf-8
import numpy as np
import pyecharts.options as opts
from pyecharts.charts import Bar, Line, HeatMap, Timeline
from pyecharts.globals import ThemeType


def draw_summary_bar(df):
    colors = ['#5793f3', '#d14a61']

    x_data = df.index.tolist()
    factor_exposure = df['factor_exposure'].tolist()
    return_attr = df['return_attr'].tolist()

    exposure_limit = int(np.max(np.abs(factor_exposure)) + 1)
    return_limit = int(np.max(np.abs(return_attr)) / 5 + 1) * 5

    bar = (
        Bar(init_opts=opts.InitOpts(width='1200px', height='600px', theme=ThemeType.WESTEROS))
        .add_xaxis(
            xaxis_data=x_data)
        .add_yaxis(
            series_name='因子暴露',
            y_axis=factor_exposure,
            label_opts=opts.LabelOpts(is_show=False),
            color=colors[0])
        .extend_axis(
            yaxis=opts.AxisOpts(
                name="收益贡献",
                type_="value",
                min_=-return_limit,
                max_=return_limit,
                split_number=6,
                axislabel_opts=opts.LabelOpts(formatter="{value} %"),
                axistick_opts=opts.AxisTickOpts(is_show=True)))
        .set_global_opts(
            title_opts=opts.TitleOpts(
                title="风格暴露及收益贡献"),
            tooltip_opts=opts.TooltipOpts(
                is_show=True, trigger="axis", axis_pointer_type="cross"),
            legend_opts=opts.LegendOpts(pos_left="center", pos_top="bottom"),
            xaxis_opts=opts.AxisOpts(
                type_="category",
                axislabel_opts={"interval": "0"},
                axistick_opts=opts.AxisTickOpts(is_show=True),
                axispointer_opts=opts.AxisPointerOpts(is_show=True, type_="shadow")),
            yaxis_opts=opts.AxisOpts(
                name="因子暴露",
                type_="value",
                min_=-exposure_limit,
                max_=exposure_limit,
                split_number=6,
                axislabel_opts=opts.LabelOpts(formatter="{value} %"),
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
            ))
    )

    bar2 = (
        Bar()
        .add_xaxis(xaxis_data=x_data)
        .add_yaxis(
            series_name="收益贡献",
            yaxis_index=1,
            y_axis=return_attr,
            label_opts=opts.LabelOpts(is_show=False))
    )

    return bar.overlap(bar2)


def draw_summary_bar_style(df):
    colors = ['#5793f3', '#d14a61']

    x_data = df.index.tolist()
    factor_exposure = df['factor_exposure'].tolist()
    return_attr = df['return_attr'].tolist()

    exposure_limit = int(np.max(np.abs(factor_exposure)) + 1)
    return_limit = int(np.max(np.abs(return_attr)) / 5 + 1) * 5

    bar = (
        Bar(init_opts=opts.InitOpts(width='1200px', height='600px', theme=ThemeType.WESTEROS))
        .add_xaxis(
            xaxis_data=x_data)
        .add_yaxis(
            series_name='因子暴露',
            y_axis=factor_exposure,
            label_opts=opts.LabelOpts(is_show=False),
            color=colors[0])
        .extend_axis(
            yaxis=opts.AxisOpts(
                name="收益贡献",
                type_="value",
                min_=-return_limit,
                max_=return_limit,
                split_number=6,
                axislabel_opts=opts.LabelOpts(formatter="{value} %"),
                axistick_opts=opts.AxisTickOpts(is_show=True)))
        .set_global_opts(
            title_opts=opts.TitleOpts(
                title="风格暴露及收益贡献"),
            tooltip_opts=opts.TooltipOpts(
                is_show=True, trigger="axis", axis_pointer_type="cross"),
            legend_opts=opts.LegendOpts(pos_left="center", pos_top="bottom"),
            xaxis_opts=opts.AxisOpts(
                type_="category",
                axislabel_opts={"interval": "0"},
                axistick_opts=opts.AxisTickOpts(is_show=True),
                axispointer_opts=opts.AxisPointerOpts(is_show=True, type_="shadow")),
            yaxis_opts=opts.AxisOpts(
                name="因子暴露",
                type_="value",
                min_=-exposure_limit,
                max_=exposure_limit,
                split_number=6,
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
            ))
    )

    bar2 = (
        Bar()
        .add_xaxis(xaxis_data=x_data)
        .add_yaxis(
            series_name="收益贡献",
            yaxis_index=1,
            y_axis=return_attr,
            label_opts=opts.LabelOpts(is_show=False))
    )

    return bar.overlap(bar2)


def draw_area_line(df, title):
    x_data = df.index.tolist()

    line = (
        Line(init_opts=opts.InitOpts(width='1200px', height='600px', theme=ThemeType.WESTEROS))
        .add_xaxis(
            xaxis_data=x_data)
        .set_global_opts(
            title_opts=opts.TitleOpts(title=title),
            tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
            legend_opts=opts.LegendOpts(pos_left="center", pos_top="bottom", legend_icon='rect'),
            yaxis_opts=opts.AxisOpts(
                type_="value",
                max_=100,
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
                axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            ),
            xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False,
                                     axistick_opts=opts.AxisTickOpts(is_show=True),),
        )
    )

    for style_factor in df.columns.tolist():
        line.add_yaxis(
            series_name=style_factor,
            stack="总量",
            y_axis=df[style_factor].tolist(),
            areastyle_opts=opts.AreaStyleOpts(opacity=0.5),
            label_opts=opts.LabelOpts(is_show=False),
        )

    return line


def draw_area_bar(df, title):
    x_data = df.index.tolist()

    bar = (
        Bar(init_opts=opts.InitOpts(width='1200px', height='600px', theme=ThemeType.WESTEROS))
        .add_xaxis(
            xaxis_data=x_data)
        .set_global_opts(
            title_opts=opts.TitleOpts(title=title),
            tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
            legend_opts=opts.LegendOpts(pos_left="center", pos_top="bottom"),
            yaxis_opts=opts.AxisOpts(
                type_="value",
                axistick_opts=opts.AxisTickOpts(is_show=True),
                axislabel_opts=opts.LabelOpts(formatter="{value} %"),
                splitline_opts=opts.SplitLineOpts(is_show=True),
            ),
            xaxis_opts=opts.AxisOpts(type_="category", axistick_opts=opts.AxisTickOpts(is_show=True)),
        )
    )

    for style_factor in df.columns.tolist():
        bar.add_yaxis(
            series_name=style_factor,
            stack="stack1",
            y_axis=df[style_factor].tolist(),
            label_opts=opts.LabelOpts(is_show=False),
            bar_width="60%"
        )

    return bar


def draw_heatmap(df, title, min_value=-4, max_value=4):
    value = [[i, j, df.values[i, j]] for i in range(df.shape[0]) for j in range(df.shape[1])]

    picture = (
        HeatMap(init_opts=opts.InitOpts(width='1200px', height='600px'))
        .add_xaxis(df.index.tolist())
        .add_yaxis(
            "风格暴露",
            df.columns.tolist(),
            value=value,
            label_opts=opts.LabelOpts(is_show=True, position='inside')
        )
        .set_global_opts(
            title_opts=opts.TitleOpts(title=title),
            visualmap_opts=opts.VisualMapOpts(min_=min_value, max_=max_value, orient="horizontal", pos_left="center")
        )
    )

    return picture


def draw_brinson_area_bar(df):
    x_data = [x[:4] + '-' + x[4:6] for x in df.index.tolist()]

    max_return = np.ceil(df.abs().max().max() * 1.2)

    bar = (
        Bar(init_opts=opts.InitOpts(width='1200px', height='600px', theme=ThemeType.MACARONS))
        .add_xaxis(
            xaxis_data=x_data)
        .set_global_opts(
            title_opts=opts.TitleOpts(title="Brinson时序归因图"),
            tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
            legend_opts=opts.LegendOpts(pos_left="center", pos_top="bottom"),
            yaxis_opts=opts.AxisOpts(
                type_="value",
                max_=max_return,
                min_=-max_return,
                axistick_opts=opts.AxisTickOpts(is_show=True),
                axislabel_opts=opts.LabelOpts(formatter="{value} %"),
                splitline_opts=opts.SplitLineOpts(is_show=True),
            ),
            xaxis_opts=opts.AxisOpts(type_="category", axistick_opts=opts.AxisTickOpts(is_show=True)),
        ).extend_axis(
            yaxis=opts.AxisOpts(
                type_="value",
                max_=max_return,
                min_=-max_return,
                axislabel_opts=opts.LabelOpts(formatter="{value} %"),
                axistick_opts=opts.AxisTickOpts(is_show=True)))
    )

    name_dict = {"asset_allo": "大类资产配置收益", "sector_allo": "行业配置收益",
                 "equity_selection": "行业选股收益", "trading": "交易收益"}

    for name in df.columns[1:].tolist():
        bar.add_yaxis(
            series_name=name_dict[name],
            stack="stack1",
            y_axis=df[name].tolist(),
            label_opts=opts.LabelOpts(is_show=False),
            bar_width="60%",
            z=0
        )

    line = (
        Line()
        .add_xaxis(xaxis_data=x_data)
        .add_yaxis(
            series_name="超额收益",
            yaxis_index=1,
            y_axis=df['active_return'],
            linestyle_opts=opts.LineStyleOpts(width=2),
            label_opts=opts.LabelOpts(is_show=True))
    )

    return bar.overlap(line)


def draw_heatmap_brinson(df):
    value = [[i, j, df.values[i, j]] for i in range(df.shape[0]) for j in range(df.shape[1])]

    value_max = df.max().max()
    value_min = df.min().min()

    picture = (
        HeatMap(init_opts=opts.InitOpts(width='1000px', height='800px'))
        .add_xaxis(df.index.tolist())
        .add_yaxis(
            "行业配置收益",
            df.columns.tolist(),
            value=value,
            label_opts=opts.LabelOpts(is_show=True, position='inside')
        )
        .set_global_opts(
            title_opts=opts.TitleOpts(title="行业配置收益时序图"),
            visualmap_opts=opts.VisualMapOpts(min_=value_min, max_=value_max, orient="horizontal", pos_left="center")
        )
    )

    return picture


def draw_weight_compare_brinson(df):
    x_data = [x[:4] + '-' + x[4:6] for x in df.index.tolist()]

    line = (
        Line(init_opts=opts.InitOpts(width='1200px', height='600px', theme=ThemeType.MACARONS))
        .add_xaxis(
            xaxis_data=x_data)
        .set_global_opts(
            title_opts=opts.TitleOpts(title="组合权益权重走势"),
            tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
            legend_opts=opts.LegendOpts(pos_left="center", pos_top="bottom"),
            yaxis_opts=opts.AxisOpts(
                type_="value",
                min_=60,
                axistick_opts=opts.AxisTickOpts(is_show=True),
                axislabel_opts=opts.LabelOpts(formatter="{value} %"),
                splitline_opts=opts.SplitLineOpts(is_show=True),
            )
        ).add_yaxis(
            series_name='基金权益权重',
            y_axis=df['portfolio_weight'].tolist(),
            label_opts=opts.LabelOpts(is_show=True),
            linestyle_opts=opts.LineStyleOpts(width=2)
        ).add_yaxis(
            series_name='基准权益权重',
            y_axis=df['benchmark_weight'].tolist(),
            label_opts=opts.LabelOpts(is_show=True),
            linestyle_opts=opts.LineStyleOpts(width=2)
        )
    )

    return line


def draw_active_exposure_series(df, title):
    bar = Bar(
        init_opts=opts.InitOpts(
            page_title=title,
            width='1200px',
            height='600px',
            theme=ThemeType.WESTEROS
        )
    ).set_global_opts(
        title_opts=opts.TitleOpts(title=title),
        legend_opts=opts.LegendOpts(legend_icon='pin', pos_top='5%'),
        xaxis_opts=opts.AxisOpts(
            type_="category",
            axistick_opts=opts.AxisTickOpts(is_show=True),
            splitline_opts=opts.SplitLineOpts(is_show=True),
            axisline_opts=opts.AxisLineOpts(on_zero_axis_index=False),
        ),
        yaxis_opts=opts.AxisOpts(
            type_="value",
            axistick_opts=opts.AxisTickOpts(is_show=True),
            splitline_opts=opts.SplitLineOpts(is_show=True),
        ),
    ).add_xaxis(
        xaxis_data=df.index.tolist()
    )

    for name in df.columns:
        bar.add_yaxis(
            series_name=name,
            y_axis=df[name].round(3).tolist(),
            label_opts=opts.LabelOpts(is_show=False),
            gap="0%"
        )

    return bar


def draw_timeline_bar(df, min_height=600):
    num = int(df.gt(0.01).sum(axis=1).mean())
    tl = Timeline(init_opts=opts.InitOpts(width='1200px', height='{}px'.format(max(25 * num, min_height))))

    for i in df.index.tolist():
        tmp = df.loc[i].dropna()
        tmp = tmp[tmp >= 0.01].sort_values()
        bar = (
            Bar()
            .add_xaxis(tmp.index.tolist())
            .add_yaxis(
                "持仓权重",
                (tmp * 100).round(2).values.tolist(),
                itemstyle_opts=opts.ItemStyleOpts(color="#37a2da"),
                label_opts=opts.LabelOpts(is_show=False))
            .reversal_axis()
            .set_global_opts(
                title_opts=opts.TitleOpts("截面持股权重 (时间点: {})".format(i)),
                xaxis_opts=opts.AxisOpts(
                    axislabel_opts=opts.LabelOpts(formatter="{value} %"),
                    splitline_opts=opts.SplitLineOpts(is_show=True, linestyle_opts=opts.LineStyleOpts(opacity=0.3)))
            )
        )
        tl.add(bar, "{}年{}月".format(i[:4], i[4:6]))

    return tl


"""
Alpha标的表现统计
"""
import os
import pandas as pd
import hbshare as hbs
import datetime
import plotly
from plotly.offline import plot as plot_ly
import plotly.graph_objs as go

plotly.offline.init_notebook_mode(connected=True)


alpha_dict = {
    "hs300": {
        "明汯稳健增长2期1号": "SK5676",
        "幻方300指数增强欣享一号": "SNL044",
        "星阔春山一号": "SQF231"
    },
    "zz500": {
        "启林": "SGY379",
        "诚奇": "SLS817",
        "幻方": "SEC185",
        "九坤": "ST9804",
        "明汯": "SEE194",
        "天演": "P20830",
        "星阔": "SNU706",
        "卓识": "SGA015",
        "衍复": "SJH866",
        "黑翼": "SEM323",
        "世纪前沿": "SGP682",
        "乾象": "P48470",
        "宽德": "SJF555",
        "因诺": "SGX346",
        "伯兄": "SQT564",
        "概率": "SQT076",
        "赫富": "SEP463",
        "量锐": "SGR954",
        "量派": "SNJ513",
        "茂源": "SJM016",
        "希格斯": "SLE315",
        "白鹭": "SQB109",
        "朋锦仲阳": "SGN254",
        "衍合": "SLG867",
        "弈倍": "SY0096"},
    "zz1000": {
        "启林": "SSU078",
        "明汯": "SGG585",
        "凡二": "SSC067",
        "衍复": "SJM688",
        "概率": "SQQ803",
        "星阔": "SQF225"
    },
    "all_market": {
        "明汯": "SSL078",
        "天演": "P22984",
        "诚奇": "SSU249",
        "星阔": "SSE288"
    },
    "market_neutral": {
        "明汯中性7号1期": "SEL756",
        "天演广全": "SLC213",
        "诚奇睿盈对冲尊享1号": "SNR622",
        "赫富对冲四号": "SEW735",
        "星阔云起1号": "SNU704",
        "茂源巴舍里耶2期": "SCV226",
        "伯兄卢比孔": "SL3246",
        "概率一号": "SNM976"
    },
    "cb": {"悬铃C号": "SEK201",
           "百奕传家一号": "SJS027",
           "艾方可转债1号": "SCK025",
           "安值福慧量化1号": "SCP765"}
}


class AlphaPerformance:
    def __init__(self, start_date, pre_date, end_date, fund_info_dict):
        self.start_date = start_date
        self.pre_date = pre_date
        self.end_date = end_date
        self.fund_info_dict = fund_info_dict
        self._load_data()

    def _load_calendar(self):
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        self.calendar_df = df[['calendarDate', 'isOpen', 'isWeekEnd', 'isMonthEnd']]

        trading_day_list = df[df['isWeekEnd'] == 1]['calendarDate'].tolist()

        return trading_day_list

    def _load_data(self):
        nav_series_dict = dict()

        for strategy_type, id_dict in self.fund_info_dict.items():
            nav_list = []
            for fund_name, fund_id in id_dict.items():
                sql_script = "SELECT a.jjdm fund_id, b.jzrq TRADEDATE, b.fqdwjz as ADJ_NAV from " \
                             "st_hedge.t_st_jjxx a, st_hedge.t_st_rhb b where a.cpfl = '4' and a.jjdm = b.jjdm " \
                             "and a.jjzt not in ('3') " \
                             "and a.jjdm = '{}' and b.jzrq >= {} and b.jzrq <= {} " \
                             "order by b.jzrq".format(fund_id, self.start_date, self.end_date)
                res = hbs.db_data_query("highuser", sql_script, page_size=5000)
                data = pd.DataFrame(res['data']).set_index('TRADEDATE')['ADJ_NAV']
                data.name = fund_name
                nav_list.append(data)

            nav_df = pd.concat(nav_list, axis=1)
            nav_series_dict[strategy_type] = nav_df.sort_index()

        self.nav_series_dict = nav_series_dict

    @staticmethod
    def _load_benchmark(benchmark_id, start_date, end_date):
        sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format(benchmark_id, start_date, end_date)
        res = hbs.db_data_query('readonly', sql_script)
        data = pd.DataFrame(res['data']).rename(columns={"TCLOSE": "benchmark"}).set_index('TRADEDATE')[['benchmark']]

        return data

    @staticmethod
    def plotly_line(df, title_text, sava_path, figsize=(1200, 500)):
        fig_width, fig_height = figsize
        data = []
        for col in df.columns:
            trace = go.Scatter(
                x=df.index.tolist(),
                y=df[col],
                name=col,
                mode="lines"
            )
            data.append(trace)

        # date_list = df.index.tolist()
        # tick_vals = [i for i in range(0, len(df), 4)]
        # tick_text = [date_list[i] for i in range(0, len(df), 4)]

        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=fig_width, height=fig_height,
            yaxis=dict(tickfont=dict(size=12), showgrid=True),
            # xaxis=dict(showgrid=True, tickvals=tick_vals, ticktext=tick_text),
            xaxis=dict(showgrid=True),
            template='plotly_white'
        )
        fig = go.Figure(data=data, layout=layout)

        plot_ly(fig, filename=sava_path, auto_open=False)

    def get_construct_result(self):
        trading_day_list = self._load_calendar()

        nan_excel = pd.DataFrame()
        file = os.path.join("D:\\量化产品跟踪\\代销及FOF标的", r"AlphaPerf_{}.xlsx".format(self.end_date))
        nan_excel.to_excel(file)
        writer = pd.ExcelWriter(file)

        # 300指增
        # nav_df = self.nav_series_dict['hs300'].reindex(trading_day_list).dropna(how='all')
        # return_df = nav_df.fillna(method='ffill').pct_change().fillna(0.)
        # benchmark_series = self._load_benchmark('000300', self.start_date, self.end_date).reindex(
        #     trading_day_list).pct_change().fillna(0.)
        # return_df = return_df.sub(benchmark_series['benchmark'], axis=0)
        # 500指增
        nav_df = self.nav_series_dict['zz500'].reindex(trading_day_list).dropna(how='all')
        # 月末日期
        tmp = pd.DataFrame({"trade_date": nav_df.index})
        tmp['month'] = tmp['trade_date'].apply(lambda x: datetime.datetime.strptime(x, '%Y%m%d').month)
        tmp['next_month'] = tmp['month'].shift(-1).fillna(0.).astype(int)
        tmp = tmp[tmp['month'] != tmp['next_month']]
        month_end = tmp['trade_date'].tolist()

        benchmark_series = self._load_benchmark('000905', self.start_date, self.end_date).reindex(
            trading_day_list).pct_change().fillna(0.)
        return_df = nav_df.fillna(method='ffill').pct_change().fillna(0.)
        return_df = return_df.sub(benchmark_series['benchmark'], axis=0)
        # 月度超额收益
        month_excess = (1 + return_df).cumprod().reindex(month_end).pct_change().dropna().sort_index().T
        month_excess.columns = [x[:6] for x in month_excess.columns]
        month_excess.to_excel(writer, sheet_name="500指增月度超额")
        # 当月超额
        week_excess = return_df[return_df.index > self.pre_date].sort_index().T
        week_excess.to_excel(writer, sheet_name="500指增当月周度超额")

        # 1000指增
        nav_df = self.nav_series_dict['zz1000'].reindex(trading_day_list).dropna(how='all')
        benchmark_series = self._load_benchmark('000852', self.start_date, self.end_date).reindex(
            trading_day_list).pct_change().fillna(0.)
        return_df = nav_df.fillna(method='ffill').pct_change().fillna(0.)
        return_df = return_df.sub(benchmark_series['benchmark'], axis=0)
        # 月度超额收益
        month_excess = (1 + return_df).cumprod().reindex(month_end).pct_change().dropna().sort_index().T
        month_excess.columns = [x[:6] for x in month_excess.columns]
        month_excess.to_excel(writer, sheet_name="1000指增月度超额")
        # 当月超额
        week_excess = return_df[return_df.index > self.pre_date].sort_index().T
        week_excess.to_excel(writer, sheet_name="1000指增当月周度超额")

        # 全市场选股
        nav_df = self.nav_series_dict['all_market'].reindex(trading_day_list).dropna(how='all')
        benchmark_series = self._load_benchmark('000985', self.start_date, self.end_date).reindex(
            trading_day_list).pct_change().fillna(0.)
        return_df = nav_df.fillna(method='ffill').pct_change().fillna(0.)
        return_df = return_df.sub(benchmark_series['benchmark'], axis=0)
        return_df = return_df[return_df.index >= '20211231']  # 全市场从22年开始看
        # 月度超额收益
        month_excess = (1 + return_df).cumprod().reindex(month_end).pct_change().dropna().sort_index().T
        month_excess.columns = [x[:6] for x in month_excess.columns]
        month_excess.to_excel(writer, sheet_name="全市场选股月度超额")
        # 当月超额
        week_excess = return_df[return_df.index > self.pre_date].sort_index().T
        week_excess.to_excel(writer, sheet_name="全市场选股当月周度超额")

        # 市场中性
        # nav_df = self.nav_series_dict['market_neutral'].reindex(trading_day_list).dropna(how='all')
        # return_df = nav_df.fillna(method='ffill').pct_change().fillna(0.)
        # adj_nav_df = (1 + return_df).cumprod()
        # self.plotly_line(
        #     adj_nav_df, "市场中性产品走势图", "D:\\量化产品跟踪\\代销及FOF标的\\市场中性走势.html", figsize=(1200, 600))
        # 可转债
        # nav_df = self.nav_series_dict['cb'].reindex(trading_day_list).dropna(how='all')
        # return_df = nav_df.fillna(method='ffill').pct_change().fillna(0.)
        # adj_nav_df = (1 + return_df).cumprod()
        # self.plotly_line(adj_nav_df, "可转债套利产品走势图", "D:\\量化产品跟踪\\代销及FOF标的\\可转债走势.html",
        #                  figsize=(1200, 600))

        writer.save()


if __name__ == '__main__':
    AlphaPerformance('20210520', '20220429', '20220527', alpha_dict).get_construct_result()


#!/usr/bin/python
#coding:utf-8

"""
@author: Meng.lv
@contact: meng.lv@howbuy.com
@software: PyCharm
@file: fund_nav_test.py
@time: 2020/6/15 10:50
"""
import os, sys
# 将hbshare根目录加入到环境变量
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(BASE_DIR)

import hbshare as hbs
import time

def test_get_fund_newest_nav():
    hbs.set_token("qwertyuisdfghjkxcvbn1000")
    data = hbs.get_fund_newest_nav_by_code('000004')
    print(data)

def get_simu_corp_list_by_keyword():
    hbs.set_token("qwertyuisdfghjkxcvbn1000")
    # data = hbs.get_simu_corp_list_by_keyword('黑石')
    data = hbs.get_prod_list_by_corp_code(80535346)
    print(data)


def get_simu_nav_by_code():
    hbs.set_token("qwertyuisdfghjkxcvbn1000")
    data, total_count = hbs.get_simu_nav_by_code("P0010", "20190101", "20191231")
    print(data)
    print(total_count)

def get_prod_valuation_by_jjdm_gzrq():
    hbs.set_token("qwertyuisdfghjkxcvbn1000")
    data = hbs.get_prod_valuation_by_jjdm_gzrq("S29494", "20150723")
    print(data)


def test_get_fund_holding():
    hbs.set_token("qwertyuisdfghjkxcvbn1000")
    data = hbs.get_fund_holding('200016', '2020-09-30')
    print(data)

def get_fund_holding_publish_date():
    hbs.set_token("qwertyuisdfghjkxcvbn1000")
    data = hbs.get_fund_holding_publish_date('200016')
    print(data)


def db_data_query():
    hbs.set_token("qwertyuisdfghjkxcvbn1000")

    sql = "select JZRQ,JJDM,HBDR,HB1Z,HB1Y,HB3Y,HB6Y from funddb.jjhb t where t.jjdm = '020005' and t.JZRQ = '20090526'"
    data = hbs.db_data_query('readonly', sql)
    print(data)

def get_authorized_product_list():
    hbs.set_token("qwertyuisdfghjkxcvbn1000")
    mytoken ="..qRibJB_jTsGjtw3Anpua9n-"
    data = hbs.get_authorized_product_list(mytoken,'false','1000')
    print(data)


if __name__ == "__main__":
    data = hbs.db_data_query('highuser',"select  jjdm,max(jzrq) as jzrq from st_hedge.t_st_sm_qjzdhc where zblb='2106' and jzrq>='20211201' and jzrq<='20211231' and zbnp!=99999 group by jjdm",timeout=300)
    print(data)
    #test_get_fund_holding()





"""
因子拥挤度
"""
import pandas as pd
import datetime
import os
import hbshare as hbs
import statsmodels.api as sm
from hbshare.fe.common.util.data_loader import get_trading_day_list
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from sqlalchemy import create_engine
from hbshare.quant.Kevin.rm_associated.config import engine_params, style_names, industry_names


class FactorCrowdCalculator:
    def __init__(self, trade_date, data_path, is_increment=1):
        self.trade_date = trade_date
        self.data_path = data_path
        self.is_increment = is_increment
        self._load_pre_date()

    def _load_pre_date(self):
        trade_dt = datetime.datetime.strptime(self.trade_date, '%Y%m%d')
        pre_date = (trade_dt - datetime.timedelta(days=200)).strftime('%Y%m%d')
        trading_day_list = get_trading_day_list(pre_date, self.trade_date)
        self.pre_date = trading_day_list[-60:][0]
        self.start_date = trading_day_list[-61:][0]

    def _calc_turnover_rate(self):
        # 换手率数据
        path = os.path.join(self.data_path, 'turnover_rate')
        listdir = os.listdir(path)
        listdir = [x for x in listdir if self.pre_date <= x.split('.')[0] <= self.trade_date]
        turnover_date = []
        for filename in listdir:
            date_t_rate = pd.read_json(os.path.join(path, '{}'.format(filename)),
                                       typ='series').to_frame('turnover_rate')
            date_t_rate.index.name = 'ticker'
            date_t_rate = date_t_rate.reset_index()
            date_t_rate['trade_date'] = filename.split('.')[0]

            turnover_date.append(date_t_rate)

        turnover_date = pd.concat(turnover_date, axis=0)
        turnover_date['ticker'] = turnover_date['ticker'].apply(lambda x: str(x).zfill(6))
        turnover_date = turnover_date[turnover_date['ticker'].str[0].isin(('0', '3', '6'))]

        turnover_df = pd.pivot_table(
            turnover_date, index='trade_date', columns='ticker', values='turnover_rate').sort_index()
        count_df = turnover_df.isnull().sum()
        included_list = count_df[count_df <= 20].index.tolist()
        turnover_df = turnover_df[included_list]

        return turnover_df.mean().to_frame('mean_to')

    def _calc_volatility(self):
        # 个股收益数据
        path = os.path.join(self.data_path, 'chg_pct')
        listdir = os.listdir(path)
        listdir = [x for x in listdir if self.pre_date <= x.split('.')[0] <= self.trade_date]
        chg_pct = []
        for filename in listdir:
            date_chg_pct = pd.read_csv(os.path.join(path, '{}'.format(filename)),
                                       dtype={'ticker': str, "tradeDate": str})
            chg_pct.append(date_chg_pct)
        chg_pct = pd.concat(chg_pct, axis=0)
        chg_pct.rename(columns={'dailyReturnReinv': 'chg_pct', 'tradeDate': 'trade_date'}, inplace=True)

        chg_pct.loc[chg_pct['chg_pct'] < -0.2, 'chg_pct'] = -0.2
        chg_pct.loc[chg_pct['chg_pct'] > 0.2, 'chg_pct'] = 0.2

        stock_return = pd.pivot_table(chg_pct, index='trade_date', columns='ticker', values='chg_pct').sort_index()
        count_df = stock_return.isnull().sum()
        included_list = count_df[count_df <= 20].index.tolist()
        stock_return = stock_return[included_list]

        return stock_return.std().to_frame('stock_vol'), stock_return

    def _calc_beta(self, stock_return):
        stock_return = stock_return.dropna(axis=1)
        # 指数收益-Wind全A
        sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format('881001', self.start_date, self.trade_date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
        index_return = data.set_index('TRADEDATE')['TCLOSE'].pct_change().dropna().reindex(stock_return.index)

        reg_x = sm.add_constant(index_return)

        beta_series = stock_return.apply(lambda y: sm.OLS(y, reg_x).fit().params[1], axis=0)

        return beta_series.to_frame('beta')

    def daily_calc(self):
        mean_to = self._calc_turnover_rate()
        stock_vol, stock_return = self._calc_volatility()
        beta = self._calc_beta(stock_return)

        result_df = mean_to.merge(stock_vol, left_index=True, right_index=True).merge(
            beta, left_index=True, right_index=True).reset_index()
        result_df['trade_date'] = self.trade_date

        return result_df

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.daily_calc()
            sql_script = "delete from factor_crowd where trade_date in ({})".format(
                ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, 'factor_crowd')
        else:
            sql_script = """
                create table factor_crowd(
                id int auto_increment primary key,
                trade_date date not null,
                ticker varchar(20),
                mean_to decimal(6, 4),
                stock_vol decimal(6, 4),
                beta decimal(6,4 )) 
            """
            create_table('factor_crowd', sql_script)
            data = self.daily_calc()
            WriteToDB().write_to_db(data, 'factor_crowd')


def get_style_factor(trade_date, factor_name):
    sql_script = "SELECT * FROM st_ashare.r_st_barra_style_factor where TRADE_DATE = '{}'".format(trade_date)
    res = hbs.db_data_query('alluser', sql_script, page_size=5000)
    style_factor = pd.DataFrame(res['data']).set_index('ticker')[factor_name]

    return style_factor


class FactorCrowdTest:
    def __init__(self, trade_date, factor_series):
        self.trade_date = trade_date
        self.factor_series = factor_series

    def run(self):
        sql_script = "SELECT * FROM factor_crowd where trade_date = '{}'".format(self.trade_date)
        engine = create_engine(engine_params)
        data = pd.read_sql(sql_script, engine)
        data['trade_date'] = data['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
        crowd_df = data.set_index('ticker')[['mean_to', 'stock_vol', 'beta']]
        df = pd.merge(crowd_df, self.factor_series.to_frame('factor'), left_index=True, right_index=True)
        crowd_list = ['mean_to', 'stock_vol', 'beta']

        short_ratio = df[df['factor'] <= df['factor'].quantile(0.1)][crowd_list].mean()
        long_ratio = df[df['factor'] >= df['factor'].quantile(0.9)][crowd_list].mean()
        ratio = long_ratio / short_ratio

        return ratio.to_frame(self.trade_date)


if __name__ == '__main__':
    # import time
    #
    # start_time = time.time()
    # FactorCrowdCalculator('20220818', r'D:\kevin\risk_model_jy\RiskModel\data\common_data', is_increment=0).get_construct_result()
    # end_time = time.time()
    # print('程序用时：%s秒' % (end_time - start_time))

    # start_date = '20220104'
    # end_date = '20220819'
    # from tqdm import tqdm
    # date_list = get_trading_day_list(start_date, end_date)
    # for date in tqdm(date_list):
    #     FactorCrowdCalculator(date, r'D:\kevin\risk_model_jy\RiskModel\data\common_data').get_construct_result()

    start_date = '20220104'
    end_date = '20220819'
    date_list = get_trading_day_list(start_date, end_date)
    from tqdm import tqdm
    ratio_list = []
    for date in tqdm(date_list):
        factor = get_style_factor(date, 'size')
        t_ratio = FactorCrowdTest(date, factor).run()
        ratio_list.append(t_ratio)

    ratio_df = pd.concat(ratio_list, axis=1)

    ratio_df.to_csv('D:\\123.csv')



from hbshare.db.ApiCfg import UrlEnum
# from ApiCfg import UrlEnum
from hbshare.base.data_pro import hb_api
# import hbshare as hbs
import pandas as pd
import time


def fieldsCheckd(kwargs, supportFields: tuple):
    fields = kwargs.get('fields')
    if fields is None or len(fields) == 0:
        # 元组转list
        kwargs['fields'] = list(supportFields)
        return
    for field in fields:
        if field not in supportFields:
            raise ValueError("field %s not supported" % field)


def commonQuery(urlEnum, pause=0.01, timeout=10, **kwargs):
    '''
        通用Api调用方法
        :param urlEnum: 指定API枚举
        :param fields: 限定查询结果列,可选,默认使用API配置的所有字段
        :return: 以fields为列的dataframe
    '''
    if urlEnum is None:
        raise ValueError("必须指定接口配置")
    if type(urlEnum) is str:
        cfg = UrlEnum.getValue(urlEnum)
    else:
        cfg = urlEnum.value
    if kwargs is None:
        kwargs = {}
    # 判断目标字段是否指定，没指定则使用接口支持字段
    fieldsCheckd(kwargs, cfg.supportFields)
    api = hb_api(timeout=timeout)
    pages = 1
    page = 1
    perPage = 1000
    data = []
    while page <= pages:
        kwargs['page'] = page
        kwargs['perPage'] = perPage
        try:
            # json传参
            post_data = cfg.parsePostData(kwargs)
            # url链接后拼接
            params = cfg.parseParams(kwargs)
        except KeyError as e:
            raise RuntimeError(
                "Use commonQuery(%s) failed,parameter %s not found" % (urlEnum, e.args[0]))

        org_js = api.query(cfg.getUrl(), method=cfg.method,
                           params=params, post_data=post_data)
        status_code = str(org_js['code'])
        if status_code != '0000':
            status = str(org_js['desc'])
            raise ValueError(status)
        if 'body' not in org_js:
            status = "未查询到数据"
            raise ValueError(status)
        body = org_js['body']
        if type(body) is list:
            # 接口不分页
            data += org_js['body']
            continue
        if 'records' in body and 'pages' in body:
            # 总页数
            pages = body['pages']
            # 页码加1
            page += 1
            data += body['records']
            time.sleep(pause)
            continue
        # 返回的是单个对象
        data.append(body)
    prod_df = pd.DataFrame(data, columns=kwargs['fields'])
    return prod_df


if __name__ == '__main__':

    data = commonQuery('FOF_ZJJLIST', startDate='20220601', endDate='20220701',
                       khdm="SSW281", jjdm='P40769', sorts=[
                           {
                               "field": "jzrq",
                               "sort": "desc"
                           }
                       ], access_token='a1df4602e9044448ab79294875f15dbd',
                       fields=['dataSource','sjly','jjdm','originalJjdm','jjmc','khdm', 'originalKhdm','khmc','jzrq'])
    print(data)



#!/usr/bin/python
#coding:utf-8

import numpy as np
import pandas as pd
from datetime import datetime
from pandas.tseries.offsets import MonthEnd
from openpyxl import load_workbook
import hbshare as hbs


class PortfolioIndexCalculator:
    def __init__(self, file_path):
        self.file_path = file_path
        self._init_params()
        self._load_data()

    def _init_params(self):
        data = pd.read_excel(self.file_path, sheet_name=0, dtype=str)
        self.benchmark_id_dict = data.filter(regex="基准_*").to_dict(orient='records')[0]
        self.start_date = data['起始日期'].values[0]
        self.end_date = data['结束日期'].values[0]
        weight_df = pd.read_excel(self.file_path, sheet_name=1)
        self.portfolio_weight_series = weight_df.set_index('产品代码')['权重']

    def _load_calendar(self):
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        return df[['calendarDate', 'isOpen', 'isWeekEnd', 'isMonthEnd']]

    def _load_fund_nav(self):
        data = []
        for fund_id in self.portfolio_weight_series.index.tolist():
            sql_script = "SELECT a.jjdm fund_id, b.jzrq tradeDate, b.fqdwjz as ADJ_NAV from " \
                         "st_hedge.t_st_jjxx a, st_hedge.t_st_rhb b where a.cpfl = '4' and a.jjdm = b.jjdm " \
                         "and a.jjzt not in ('3') " \
                         "and a.jjdm = '{}' and b.jzrq >= {} and b.jzrq <= {} " \
                         "order by b.jzrq".format(fund_id, self.start_date, self.end_date)
            res = hbs.db_data_query("highuser", sql_script, page_size=5000)
            df = pd.DataFrame(res['data'])
            data.append(df)
        data = pd.concat(data)

        data.rename(columns={"FUND_ID": "fund_id", "ADJ_NAV": "adj_nav"}, inplace=True)

        return data

    def _load_benchmark_data(self):
        data = []
        for key, value in self.benchmark_id_dict.items():
            sql_script = "SELECT JYRQ, ZQDM, SPJG FROM funddb.ZSJY WHERE ZQDM = '{}' and " \
                         "JYRQ >= {} and JYRQ <= {}".format(value, self.start_date, self.end_date)
            res = hbs.db_data_query('readonly', sql_script)
            df = pd.DataFrame(res['data'])
            df = df.rename(columns={"JYRQ": "tradeDate", "SPJG": "close"}).set_index('tradeDate')['close']
            df.name = key
            data.append(df)

        data = pd.concat(data, axis=1)

        return data

    def _load_data(self):
        # calendar
        self.calendar_df = self._load_calendar()
        weekend_list = self.calendar_df[
            (self.calendar_df['isWeekEnd'] == 1) & (self.calendar_df['isOpen'] == 1)]['calendarDate'].tolist()
        # fund_nav
        fund_nav_df = self._load_fund_nav()
        nav_df = pd.pivot_table(fund_nav_df, index='tradeDate', columns='fund_id', values='adj_nav')
        natural_days = [x.strftime("%Y%m%d") for x in pd.date_range(nav_df.index[0], nav_df.index[-1])]
        nav_df = nav_df.reindex(natural_days).interpolate().reindex(weekend_list)
        self.nav_df = nav_df / nav_df.iloc[0]
        # benchmark
        self.benchmark_nav_df = self._load_benchmark_data()

    @staticmethod
    def cal_annual_return(return_series):
        T = len(return_series)
        annual_return = (1 + return_series).prod() ** (52 / T) - 1

        return annual_return

    @staticmethod
    def cal_annual_volatility(return_series):
        vol = return_series.std() * np.sqrt(52)

        return vol

    @staticmethod
    def cal_max_drawdown(nav_series):
        drawdown_series = nav_series / (nav_series.cummax()) - 1

        return drawdown_series.min()

    def cal_sharpe_ratio(self, return_series, rf):
        annual_return = self.cal_annual_return(return_series)
        vol = self.cal_annual_volatility(return_series)
        sharpe_ratio = (annual_return - rf) / vol

        return sharpe_ratio

    @staticmethod
    def cal_month_ret(return_series):
        name = return_series.name
        df = return_series.copy().reset_index()
        df['month'] = df['tradeDate'].apply(lambda x: x[:6])
        month_ret = df.groupby('month')[name].apply(lambda x: (1 + x).prod() - 1)

        max_rise = month_ret.max() if month_ret.max() > 0 else 0
        max_decline = month_ret.min() if month_ret.min() < 0 else 0
        win_ratio = len(month_ret[month_ret > 0]) / month_ret.shape[0]

        return max_rise, max_decline, win_ratio

    def cal_performance_index(self, return_series, nav_series):
        annual_return = self.cal_annual_return(return_series)
        annual_vol = self.cal_annual_volatility(return_series)
        max_drawdown = self.cal_max_drawdown(nav_series)
        sharpe_ratio = self.cal_sharpe_ratio(return_series, rf=0.015)
        calmar_ratio = annual_return / np.abs(max_drawdown)
        max_rise, max_decline, win_ratio = self.cal_month_ret(return_series)

        df = pd.DataFrame(index=["指标", "年化收益", "年化波动", "最大回撤", "夏普比率", "卡玛比率",
                                 "最大月度涨幅", "最大月度跌幅", "月胜率"],
                          data=[return_series.name, annual_return, annual_vol,
                                max_drawdown, sharpe_ratio, calmar_ratio,
                                max_rise, max_decline, win_ratio]).T

        return df

    @staticmethod
    def get_pre_N_date(dt, N):
        pre_dt = dt - MonthEnd(N)
        day = min(pre_dt.day, dt.day)
        actual_pre_date = datetime(pre_dt.year, pre_dt.month, day).strftime('%Y%m%d')

        return actual_pre_date

    def write_to_excel(self, output):
        writer = pd.ExcelWriter(self.file_path, engine='openpyxl')
        book = load_workbook(self.file_path)
        writer.book = book

        fof_nav = output["组合净值"].reset_index().rename(columns={"tradeDate": "日期", "组合": "FOF组合"})
        fof_nav.to_excel(writer, "组合净值", index=False)
        fof_ret = output["组合收益"].reset_index().rename(columns={"tradeDate": "日期", "组合": "FOF组合"})
        fof_ret.to_excel(writer, "组合收益", index=False)
        output['业绩指标'].to_excel(writer, "业绩风险指标", index=False)
        output['区间收益'].to_excel(writer, "业绩风险指标", startrow=10, index=False)
        output['年度收益'].to_excel(writer, "业绩风险指标", startrow=20, index=False)
        output['子基金相关性'].to_excel(writer, "业绩风险指标", startrow=30)

        writer.save()
        writer.close()

    def get_construct_result(self):
        # 组合收益数据
        portfolio_nav = self.nav_df.dot(self.portfolio_weight_series)
        benchmark_nav = self.benchmark_nav_df.reindex(portfolio_nav.index)
        benchmark_nav /= benchmark_nav.iloc[0]
        total_nav_df = pd.merge(
            portfolio_nav.to_frame('组合'), benchmark_nav, left_index=True, right_index=True)
        total_return_df = total_nav_df.pct_change().dropna()
        # 业绩指标
        performance_df = []
        for name in total_return_df.columns:
            df = self.cal_performance_index(total_return_df[name], total_nav_df[name])
            performance_df.append(df)
        performance_df = pd.concat(performance_df)
        # 区间收益
        end_dt = datetime.strptime(total_return_df.index[-1], "%Y%m%d")
        year_start = total_return_df.index[-1][:4] + '0101'
        period_ret1 = (1 + total_return_df[total_return_df.index > year_start]).prod() - 1
        pre_1_month = self.get_pre_N_date(end_dt, 1)
        period_ret_1M = (1 + total_return_df[total_return_df.index > pre_1_month]).prod() - 1
        pre_3_month = self.get_pre_N_date(end_dt, 3)
        period_ret_3M = (1 + total_return_df[total_return_df.index >= pre_3_month]).prod() - 1
        pre_6_month = self.get_pre_N_date(end_dt, 6)
        period_ret_6M = (1 + total_return_df[total_return_df.index >= pre_6_month]).prod() - 1
        pre_12_month = self.get_pre_N_date(end_dt, 12)
        period_ret_12M = (1 + total_return_df[total_return_df.index >= pre_12_month]).prod() - 1
        all_ret = (1 + total_return_df).prod() - 1
        period_ret = pd.concat(
            [period_ret1, period_ret_1M, period_ret_3M, period_ret_6M, period_ret_12M, all_ret], axis=1)
        period_ret.reset_index(inplace=True)
        period_ret.columns = ['指标', '今年以来', '近一月', '近3月', '近6月', '近一年', '起始以来']
        # 年度收益
        tmp = total_return_df.copy()
        tmp['trade_date'] = tmp.index
        tmp['year'] = tmp['trade_date'].apply(lambda x: datetime.strptime(x, '%Y%m%d').year)
        year_ret = tmp.groupby('year').apply(
            lambda x: (1 + x[[a for a in x.columns if a not in ['trade_date', 'year']]]).prod() - 1).T.reset_index()
        year_ret.rename(columns={"index": "指标"}, inplace=True)
        # 子基金相关性
        fund_corr = self.nav_df.pct_change().dropna().corr()

        output = {"组合收益": total_return_df,
                  "组合净值": total_nav_df,
                  "业绩指标": performance_df,
                  "区间收益": period_ret,
                  "年度收益": year_ret,
                  "子基金相关性": fund_corr}

        self.write_to_excel(output)

        return output


if __name__ == '__main__':
    results = PortfolioIndexCalculator("D:\\kevin\\好买30投前指标模板.xlsx").get_construct_result()



# -*- coding:utf-8 -*-
"""
Created on 2020/6/15
@author: Meng Lv
@group: Howbuy FDC
@contact: meng.lv@howbuy.com
"""
VERSION = '1.0.1'
P_TYPE = {'http': 'http://', 'ftp': 'ftp://', 'https': 'https://'}
FORMAT = lambda x: '%.2f' % x
FORMAT4 = lambda x: '%.4f' % x
DOMAINS = {'hbcgi': 'data.howbuy.com', 's': 's.howbuy.com','ams':'ams-data.intelnal.howbuy.com'}

##########################################################################
# 基金数据列名
HOWBUY_SIMU_CORP = ['tsname', 'tsshortName', 'tscode']
HOWBUY_SIMU_PROD_LIST = ['fundCode', 'fundName', 'hb1n','jjjz', 'jzrq', 'nhbdl', 'nhsyl']
HOWBUY_SIMU_NAV = ['jzrq', 'jjjz', 'ljjz','fqdwjz', 'jjjzStr', 'ljjzStr']
HOWBUY_SIMU_VALUATION = ['jjdm','gzrq','kmdm','kmmc','bz','hl','sl','dwcb','cb','cbzjz','hqssj','sz','szzjz','gzzz','tpxx','qyxx']

#test
##########################################################################
# 数据源URL
HOWBUY_SIMU_CORP_SEARCH = "%s%s/simu_searchdata_new.do?q=%s&fq=600&type=1"
HOWBUY_SIMU_GET_PROD_LIST_BY_CORP_CODE = "%s%s/cgi/simu/v634/company.json?jgdm=%s"
HOWBUY_SIMU_GET_NAV_LIST_BY_CODE = "%s%s/cgi/fund/historyfundnetvalueofpage.json?fundCode=%s&isPrivate=1&startDate=%s&endDate=%s&pageCount=%s&pageNum=%s"
# 私募估值信息
HOWBUY_SIMU_GET_VALUATION_LIST_BY_JJDM_GZRQ = "%s%s/data/query/guzhi/info?gzrq=%s&jjdm=%s"

##########################################################################
DATA_GETTING_TIPS = '[Getting data:]'
DATA_GETTING_FLAG = '#'
DATA_ROWS_TIPS = '%s rows data found.Please wait for a moment.'
DATA_INPUT_ERROR_MSG = 'date input error.'
NETWORK_URL_ERROR_MSG = '获取失败，请检查网络和URL'
DATE_CHK_MSG = '年度输入错误：请输入1989年以后的年份数字，格式：YYYY'
DATE_CHK_Q_MSG = '季度输入错误：请输入1、2、3或4数字'
TOP_PARAS_MSG = 'top有误，请输入整数或all.'
LHB_MSG = '周期输入有误，请输入数字5、10、30或60'

OFT_MSG = u'开放型基金类型输入有误，请输入all、equity、mix、bond、monetary、qdii'

DICT_NAV_EQUITY = {
    'fbrq': 'date',
    'jjjz': 'value',
    'ljjz': 'total',
    'change': 'change'
}

DICT_NAV_MONETARY = {
    'fbrq': 'date',
    'nhsyl': 'value',
    'dwsy': 'total',
    'change': 'change'
}

import sys
PY3 = (sys.version_info[0] >= 3)


def _write_head():
    sys.stdout.write(DATA_GETTING_TIPS)
    sys.stdout.flush()


def _write_console():
    sys.stdout.write(DATA_GETTING_FLAG)
    sys.stdout.flush()


def _write_tips(tip):
    sys.stdout.write(DATA_ROWS_TIPS % tip)
    sys.stdout.flush()


def _write_msg(msg):
    sys.stdout.write(msg)
    sys.stdout.flush()


def _check_input(year, quarter):
    if isinstance(year, str) or year < 1989:
        raise TypeError(DATE_CHK_MSG)
    elif quarter is None or isinstance(quarter, str) or quarter not in [1, 2, 3, 4]:
        raise TypeError(DATE_CHK_Q_MSG)
    else:
        return True



TIME_SOLT = [("2000","前一周"),
            ("2007","近7天(1周）"),
            ("2014","近14天(2周)"),
            ("2028","近28天(4周)"),
            ("2101","近1月"),
            ("2102","近2月"),
            ("2103","近2月"),
            ("2106","近6月"),
            ("2118","近18月"),
            ("2201","近1年"),
            ("2202","近2年"),
            ("2203","近3年"),
            ("2205","近5年"),
            ("2210","近10年"),
            ("2998","今年以来"),
            ("2999","成立以来")]

TIME_DICT = {"DAY":"jzrq", "WEEK": "tjrq", "MONTH": "tjyf", "YEAR": "tjnf"}

def sql(k):
    key, dim_time = split_key(k)
    df = get_all_indexes()
    # print(df.loc["VOLATILITY"]['MONTH'])
    # print(df.index.isin(['VOLATILITY_RANK']))
    if dim_time not in list(TIME_DICT.keys()):
        raise KeyError(k ," 请检查时间维定义是否正确 ")

    if key not in df.index:
        raise KeyError(key ," 该指标不存在 ")

    if dim_time not in df.columns:
        raise KeyError(dim_time ," 配置中不存在该时间维度 ")

    if df.loc[key][dim_time] == "":
        raise KeyError(k, " 请和DB团队确认该指标是否计算!")

    print(str(df.loc[key]['NAME_CN']))
    return "select * from st_hedge."+str(df.loc[key][dim_time])+" a where a.jjdm ='%s' and "+ TIME_DICT[dim_time] + "='%s'"

def get_all_indexes():
    df = pd.read_csv("simu/simu.csv")
    df = df.set_index('KEY')
    #df = df.fillna(None)
    df = df.replace(np.nan, '', regex=True)
    return df

def split_key(k):
    last_idx = k.rfind("_")
    key = k[0:last_idx]
    dim_time = k[last_idx + 1:len(k)]
    return key, dim_time

def check_dim_time(k, dim_time):
    key, dt = split_key(k)

    df = get_all_indexes()
    # print(df.loc["VOLATILITY"]['MONTH'])
    # print(df.index.isin(['VOLATILITY_RANK']))
    if dt not in list(TIME_DICT.keys()):
        raise KeyError(k, " 请设置正确的时间维度格式 ")
    if dt == 'DAY' and len(dim_time) != 8:
        raise KeyError(dim_time, " 日频指标请确定时间维度格式：yyyymmdd ")
    if dt == 'WEEK' and len(dim_time) != 8:
        raise KeyError(dim_time, " 周频指标请确定时间维度格式：yyyymmdd ")
    if dt == 'MONTH' and len(dim_time) != 6:
        raise KeyError(dim_time, " 月频指标请确定时间维度格式：yyyymm ")
    if dt == 'YEAR' and len(dim_time) != 4:
        raise KeyError(dim_time, " 年度指标请确定时间维度格式：yyyy ")
